from typing import Final

from braintree.attribute_getter import AttributeGetter
from braintree.error_result import ErrorResult
from braintree.resource_collection import ResourceCollection
from braintree.successful_result import SuccessfulResult
from braintree.us_bank_account import UsBankAccount

class UsBankAccountVerification(AttributeGetter):
    class Status:
        Failed: Final = "failed"
        GatewayRejected: Final = "gateway_rejected"
        ProcessorDeclined: Final = "processor_declined"
        Unrecognized: Final = "unrecognized"
        Verified: Final = "verified"
        Pending: Final = "pending"

    class VerificationMethod:
        NetworkCheck: Final = "network_check"
        IndependentCheck: Final = "independent_check"
        InstantVerificationAccountValidation: Final = "instant_verification_account_validation"
        TokenizedCheck: Final = "tokenized_check"
        MicroTransfers: Final = "micro_transfers"

    class VerificationAddOns:
        CustomerVerification: Final = "customer_verification"

    us_bank_account: UsBankAccount | None
    def __init__(self, gateway, attributes) -> None: ...
    @staticmethod
    def confirm_micro_transfer_amounts(verification_id: str, amounts) -> SuccessfulResult | ErrorResult | None: ...
    @staticmethod
    def find(verification_id: str) -> UsBankAccountVerification: ...
    @staticmethod
    def search(*query) -> ResourceCollection: ...
    def __eq__(self, other: object) -> bool: ...



from _typeshed import SupportsFlush, SupportsRead, SupportsReadline, SupportsWrite
from typing import Any, type_check_only

import gdb

from .server import _JSONValue
from .startup import DAPQueue

@type_check_only
class _SupportsReadAndReadlineBytes(SupportsRead[bytes], SupportsReadline[bytes]): ...

@type_check_only
class _SupportsWriteAndFlushBytes(SupportsWrite[bytes], SupportsFlush): ...

def read_json(stream: _SupportsReadAndReadlineBytes) -> Any: ...  # returns result of json.loads
def start_json_writer(stream: _SupportsWriteAndFlushBytes, queue: DAPQueue[_JSONValue]) -> gdb.Thread: ...



from unidiff.patch import (
    DEFAULT_ENCODING as DEFAULT_ENCODING,
    LINE_TYPE_ADDED as LINE_TYPE_ADDED,
    LINE_TYPE_CONTEXT as LINE_TYPE_CONTEXT,
    LINE_TYPE_REMOVED as LINE_TYPE_REMOVED,
    Hunk as Hunk,
    PatchedFile as PatchedFile,
    PatchSet as PatchSet,
    UnidiffParseError as UnidiffParseError,
)

VERSION: str



from _typeshed import Incomplete, StrPath
from collections.abc import Iterable, Iterator, MutableMapping
from importlib import metadata
from typing import Literal, TypeVar, overload

from . import Command, SetuptoolsDeprecationWarning
from ._distutils.cmd import Command as _Command
from ._distutils.dist import Distribution as _Distribution
from .command.alias import alias
from .command.bdist_egg import bdist_egg
from .command.bdist_rpm import bdist_rpm
from .command.bdist_wheel import bdist_wheel
from .command.build import build
from .command.build_clib import build_clib
from .command.build_ext import build_ext
from .command.build_py import build_py
from .command.develop import develop
from .command.dist_info import dist_info
from .command.easy_install import easy_install
from .command.editable_wheel import editable_wheel
from .command.egg_info import egg_info
from .command.install import install
from .command.install_egg_info import install_egg_info
from .command.install_lib import install_lib
from .command.install_scripts import install_scripts
from .command.rotate import rotate
from .command.saveopts import saveopts
from .command.sdist import sdist
from .command.setopt import setopt

_CommandT = TypeVar("_CommandT", bound=_Command)

__all__ = ["Distribution"]

class Distribution(_Distribution):
    include_package_data: bool | None
    exclude_package_data: dict[str, list[str]] | None
    src_root: str | None
    dependency_links: list[str]
    setup_requires: list[str]
    def __init__(self, attrs: MutableMapping[str, Incomplete] | None = None) -> None: ...
    def parse_config_files(self, filenames: Iterable[StrPath] | None = None, ignore_option_errors: bool = False) -> None: ...
    def fetch_build_eggs(self, requires: str | Iterable[str]) -> list[metadata.Distribution]: ...
    def get_egg_cache_dir(self) -> str: ...
    def fetch_build_egg(self, req): ...
    # NOTE: Commands that setuptools doesn't re-expose are considered deprecated (they must be imported from distutils directly)
    # So we're not listing them here. This list comes directly from the setuptools/command folder. Minus the test command.
    @overload  # type: ignore[override]
    def get_command_obj(self, command: Literal["alias"], create: Literal[1, True] = 1) -> alias: ...
    @overload
    def get_command_obj(self, command: Literal["bdist_egg"], create: Literal[1, True] = 1) -> bdist_egg: ...
    @overload
    def get_command_obj(self, command: Literal["bdist_rpm"], create: Literal[1, True] = 1) -> bdist_rpm: ...  # type: ignore[overload-overlap]
    @overload
    def get_command_obj(self, command: Literal["bdist_wheel"], create: Literal[1, True] = 1) -> bdist_wheel: ...
    @overload
    def get_command_obj(self, command: Literal["build"], create: Literal[1, True] = 1) -> build: ...  # type: ignore[overload-overlap]
    @overload
    def get_command_obj(self, command: Literal["build_clib"], create: Literal[1, True] = 1) -> build_clib: ...  # type: ignore[overload-overlap]
    @overload
    def get_command_obj(self, command: Literal["build_ext"], create: Literal[1, True] = 1) -> build_ext: ...  # type: ignore[overload-overlap]
    @overload
    def get_command_obj(self, command: Literal["build_py"], create: Literal[1, True] = 1) -> build_py: ...  # type: ignore[overload-overlap]
    @overload
    def get_command_obj(self, command: Literal["develop"], create: Literal[1, True] = 1) -> develop: ...
    @overload
    def get_command_obj(self, command: Literal["dist_info"], create: Literal[1, True] = 1) -> dist_info: ...  # type: ignore[overload-overlap]
    @overload
    def get_command_obj(self, command: Literal["easy_install"], create: Literal[1, True] = 1) -> easy_install: ...
    @overload
    def get_command_obj(self, command: Literal["editable_wheel"], create: Literal[1, True] = 1) -> editable_wheel: ...
    @overload
    def get_command_obj(self, command: Literal["egg_info"], create: Literal[1, True] = 1) -> egg_info: ...
    @overload
    def get_command_obj(self, command: Literal["install"], create: Literal[1, True] = 1) -> install: ...  # type: ignore[overload-overlap]
    @overload
    def get_command_obj(self, command: Literal["install_egg_info"], create: Literal[1, True] = 1) -> install_egg_info: ...
    @overload
    def get_command_obj(self, command: Literal["install_lib"], create: Literal[1, True] = 1) -> install_lib: ...  # type: ignore[overload-overlap]
    @overload
    def get_command_obj(self, command: Literal["install_scripts"], create: Literal[1, True] = 1) -> install_scripts: ...  # type: ignore[overload-overlap]
    @overload
    def get_command_obj(self, command: Literal["rotate"], create: Literal[1, True] = 1) -> rotate: ...
    @overload
    def get_command_obj(self, command: Literal["saveopts"], create: Literal[1, True] = 1) -> saveopts: ...
    @overload
    def get_command_obj(self, command: Literal["sdist"], create: Literal[1, True] = 1) -> sdist: ...  # type: ignore[overload-overlap]
    @overload
    def get_command_obj(self, command: Literal["setopt"], create: Literal[1, True] = 1) -> setopt: ...
    @overload
    def get_command_obj(self, command: str, create: Literal[1, True] = 1) -> Command: ...
    # Not replicating the overloads for "Command | None", user may use "isinstance"
    @overload
    def get_command_obj(self, command: str, create: Literal[0, False]) -> Command | None: ...
    @overload
    def get_command_class(self, command: Literal["alias"]) -> type[alias]: ...
    @overload
    def get_command_class(self, command: Literal["bdist_egg"]) -> type[bdist_egg]: ...
    @overload
    def get_command_class(self, command: Literal["bdist_rpm"]) -> type[bdist_rpm]: ...  # type: ignore[overload-overlap]
    @overload
    def get_command_class(self, command: Literal["bdist_wheel"]) -> type[bdist_wheel]: ...
    @overload
    def get_command_class(self, command: Literal["build"]) -> type[build]: ...  # type: ignore[overload-overlap]
    @overload
    def get_command_class(self, command: Literal["build_clib"]) -> type[build_clib]: ...  # type: ignore[overload-overlap]
    @overload
    def get_command_class(self, command: Literal["build_ext"]) -> type[build_ext]: ...  # type: ignore[overload-overlap]
    @overload
    def get_command_class(self, command: Literal["build_py"]) -> type[build_py]: ...  # type: ignore[overload-overlap]
    @overload
    def get_command_class(self, command: Literal["develop"]) -> type[develop]: ...
    @overload
    def get_command_class(self, command: Literal["dist_info"]) -> type[dist_info]: ...  # type: ignore[overload-overlap]
    @overload
    def get_command_class(self, command: Literal["easy_install"]) -> type[easy_install]: ...
    @overload
    def get_command_class(self, command: Literal["editable_wheel"]) -> type[editable_wheel]: ...
    @overload
    def get_command_class(self, command: Literal["egg_info"]) -> type[egg_info]: ...
    @overload
    def get_command_class(self, command: Literal["install"]) -> type[install]: ...  # type: ignore[overload-overlap]
    @overload
    def get_command_class(self, command: Literal["install_egg_info"]) -> type[install_egg_info]: ...
    @overload
    def get_command_class(self, command: Literal["install_lib"]) -> type[install_lib]: ...  # type: ignore[overload-overlap]
    @overload
    def get_command_class(self, command: Literal["install_scripts"]) -> type[install_scripts]: ...  # type: ignore[overload-overlap]
    @overload
    def get_command_class(self, command: Literal["rotate"]) -> type[rotate]: ...
    @overload
    def get_command_class(self, command: Literal["saveopts"]) -> type[saveopts]: ...
    @overload
    def get_command_class(self, command: Literal["sdist"]) -> type[sdist]: ...  # type: ignore[overload-overlap]
    @overload
    def get_command_class(self, command: Literal["setopt"]) -> type[setopt]: ...
    @overload
    def get_command_class(self, command: str) -> type[Command]: ...
    @overload  # type: ignore[override]
    def reinitialize_command(self, command: Literal["alias"], reinit_subcommands: bool = False) -> alias: ...
    @overload
    def reinitialize_command(self, command: Literal["bdist_egg"], reinit_subcommands: bool = False) -> bdist_egg: ...
    @overload
    def reinitialize_command(self, command: Literal["bdist_rpm"], reinit_subcommands: bool = False) -> bdist_rpm: ...  # type: ignore[overload-overlap]
    @overload
    def reinitialize_command(self, command: Literal["bdist_wheel"], reinit_subcommands: bool = False) -> bdist_wheel: ...
    @overload
    def reinitialize_command(self, command: Literal["build"], reinit_subcommands: bool = False) -> build: ...  # type: ignore[overload-overlap]
    @overload
    def reinitialize_command(self, command: Literal["build_clib"], reinit_subcommands: bool = False) -> build_clib: ...  # type: ignore[overload-overlap]
    @overload
    def reinitialize_command(self, command: Literal["build_ext"], reinit_subcommands: bool = False) -> build_ext: ...  # type: ignore[overload-overlap]
    @overload
    def reinitialize_command(self, command: Literal["build_py"], reinit_subcommands: bool = False) -> build_py: ...  # type: ignore[overload-overlap]
    @overload
    def reinitialize_command(self, command: Literal["develop"], reinit_subcommands: bool = False) -> develop: ...
    @overload
    def reinitialize_command(self, command: Literal["dist_info"], reinit_subcommands: bool = False) -> dist_info: ...  # type: ignore[overload-overlap]
    @overload
    def reinitialize_command(self, command: Literal["easy_install"], reinit_subcommands: bool = False) -> easy_install: ...
    @overload
    def reinitialize_command(self, command: Literal["editable_wheel"], reinit_subcommands: bool = False) -> editable_wheel: ...
    @overload
    def reinitialize_command(self, command: Literal["egg_info"], reinit_subcommands: bool = False) -> egg_info: ...
    @overload
    def reinitialize_command(self, command: Literal["install"], reinit_subcommands: bool = False) -> install: ...  # type: ignore[overload-overlap]
    @overload
    def reinitialize_command(
        self, command: Literal["install_egg_info"], reinit_subcommands: bool = False
    ) -> install_egg_info: ...
    @overload
    def reinitialize_command(self, command: Literal["install_lib"], reinit_subcommands: bool = False) -> install_lib: ...  # type: ignore[overload-overlap]
    @overload
    def reinitialize_command(self, command: Literal["install_scripts"], reinit_subcommands: bool = False) -> install_scripts: ...  # type: ignore[overload-overlap]
    @overload
    def reinitialize_command(self, command: Literal["rotate"], reinit_subcommands: bool = False) -> rotate: ...
    @overload
    def reinitialize_command(self, command: Literal["saveopts"], reinit_subcommands: bool = False) -> saveopts: ...
    @overload
    def reinitialize_command(self, command: Literal["sdist"], reinit_subcommands: bool = False) -> sdist: ...  # type: ignore[overload-overlap]
    @overload
    def reinitialize_command(self, command: Literal["setopt"], reinit_subcommands: bool = False) -> setopt: ...
    @overload
    def reinitialize_command(self, command: str, reinit_subcommands: bool = False) -> Command: ...
    @overload
    def reinitialize_command(self, command: _CommandT, reinit_subcommands: bool = False) -> _CommandT: ...
    def include(self, **attrs) -> None: ...
    def exclude_package(self, package: str) -> None: ...
    def has_contents_for(self, package: str) -> bool: ...
    def exclude(self, **attrs) -> None: ...
    def get_cmdline_options(self) -> dict[str, dict[str, str | None]]: ...
    def iter_distribution_names(self) -> Iterator[str]: ...
    def handle_display_options(self, option_order): ...

class DistDeprecationWarning(SetuptoolsDeprecationWarning): ...



from _typeshed import Incomplete
from typing import Literal, TextIO

from antlr4.atn.ATNDeserializationOptions import ATNDeserializationOptions as ATNDeserializationOptions
from antlr4.atn.ATNDeserializer import ATNDeserializer as ATNDeserializer
from antlr4.atn.ParserATNSimulator import ParserATNSimulator
from antlr4.BufferedTokenStream import TokenStream as TokenStream
from antlr4.CommonTokenFactory import TokenFactory as TokenFactory
from antlr4.error.Errors import (
    RecognitionException as RecognitionException,
    UnsupportedOperationException as UnsupportedOperationException,
)
from antlr4.error.ErrorStrategy import DefaultErrorStrategy as DefaultErrorStrategy
from antlr4.InputStream import InputStream as InputStream
from antlr4.Lexer import Lexer as Lexer
from antlr4.ParserRuleContext import ParserRuleContext as ParserRuleContext
from antlr4.Recognizer import Recognizer as Recognizer
from antlr4.RuleContext import RuleContext as RuleContext
from antlr4.Token import Token as Token
from antlr4.tree.ParseTreePattern import ParseTreePattern
from antlr4.tree.ParseTreePatternMatcher import ParseTreePatternMatcher as ParseTreePatternMatcher
from antlr4.tree.Tree import ErrorNode as ErrorNode, ParseTreeListener as ParseTreeListener, TerminalNode as TerminalNode

class TraceListener(ParseTreeListener):
    __slots__ = "_parser"
    def __init__(self, parser) -> None: ...
    def enterEveryRule(self, ctx) -> None: ...
    def visitTerminal(self, node) -> None: ...
    def visitErrorNode(self, node) -> None: ...
    def exitEveryRule(self, ctx) -> None: ...

class Parser(Recognizer):
    __slots__ = (
        "_input",
        "_output",
        "_errHandler",
        "_precedenceStack",
        "_ctx",
        "buildParseTrees",
        "_tracer",
        "_parseListeners",
        "_syntaxErrors",
    )
    _input: TokenStream
    _output: TextIO
    _errHandler: DefaultErrorStrategy
    _precedenceStack: list[int]
    _ctx: ParserRuleContext | None
    _tracer: TraceListener | None
    _parseListeners: list[ParseTreeListener]
    _syntaxErrors: int
    _interp: ParserATNSimulator
    bypassAltsAtnCache: dict[Incomplete, Incomplete]
    buildParseTrees: bool
    def __init__(self, input: TokenStream, output: TextIO = ...) -> None: ...
    def reset(self) -> None: ...
    def match(self, ttype: int) -> Token: ...
    def matchWildcard(self) -> Token: ...
    def getParseListeners(self) -> list[ParseTreeListener]: ...
    def addParseListener(self, listener: ParseTreeListener) -> None: ...
    def removeParseListener(self, listener: ParseTreeListener) -> None: ...
    def removeParseListeners(self) -> None: ...
    def triggerEnterRuleEvent(self) -> None: ...
    def triggerExitRuleEvent(self) -> None: ...
    def getNumberOfSyntaxErrors(self) -> int: ...
    def getTokenFactory(self) -> TokenFactory: ...
    def setTokenFactory(self, factory: TokenFactory) -> None: ...
    def getATNWithBypassAlts(self): ...
    def compileParseTreePattern(self, pattern: str, patternRuleIndex: int, lexer: Lexer | None = None) -> ParseTreePattern: ...
    def getInputStream(self) -> InputStream: ...
    def setInputStream(self, input: InputStream) -> None: ...
    def getTokenStream(self) -> TokenStream: ...
    def setTokenStream(self, input: TokenStream) -> None: ...
    def getCurrentToken(self) -> Token | None: ...
    def notifyErrorListeners(
        self, msg: str, offendingToken: Token | None = None, e: RecognitionException | None = None
    ) -> None: ...
    def consume(self) -> None: ...
    def addContextToParseTree(self) -> None: ...
    state: int
    def enterRule(self, localctx: ParserRuleContext, state: int, ruleIndex: int) -> None: ...
    def exitRule(self) -> None: ...
    def enterOuterAlt(self, localctx: ParserRuleContext, altNum: int) -> None: ...
    def getPrecedence(self) -> int: ...
    def enterRecursionRule(self, localctx: ParserRuleContext, state: int, ruleIndex: int, precedence: int) -> None: ...
    def pushNewRecursionContext(self, localctx: ParserRuleContext, state: int, ruleIndex: int) -> None: ...
    def unrollRecursionContexts(self, parentCtx: ParserRuleContext) -> None: ...
    def getInvokingContext(self, ruleIndex: int) -> RuleContext | None: ...
    def precpred(self, localctx: RuleContext, precedence: int) -> bool: ...
    def inContext(self, context: str) -> Literal[False]: ...
    def isExpectedToken(self, symbol: int) -> bool: ...
    def getExpectedTokens(self): ...
    def getExpectedTokensWithinCurrentRule(self): ...
    def getRuleIndex(self, ruleName: str) -> int: ...
    def getRuleInvocationStack(self, p: RuleContext | None = None) -> list[str]: ...
    def getDFAStrings(self) -> list[str]: ...
    def dumpDFA(self) -> None: ...
    def getSourceName(self) -> str: ...
    def setTrace(self, trace: bool) -> None: ...



from typing import Generic, TypeVar

_T = TypeVar("_T")

class TypeChecker(Generic[_T]):
    def __init__(self, *acceptable_types: _T): ...
    def CheckValue(self, proposed_value: _T) -> _T: ...

class TypeCheckerWithDefault(TypeChecker[_T]):
    def __init__(self, default_value: _T, *acceptable_types: _T): ...
    def DefaultValue(self) -> _T: ...



import asyncio
import socket
from _typeshed import Incomplete, Unused
from _typeshed.wsgi import WSGIEnvironment
from collections.abc import Awaitable, Callable
from ssl import SSLContext
from typing import Any, Literal, TypedDict, type_check_only

from wsproto import ConnectionType, WSConnection
from wsproto.events import Request
from wsproto.frame_protocol import CloseReason

from .asgi import WebSocketASGI, _SocketDataBase, _SocketDataBytes, _SocketDataProtocol, _SocketDataStr

class AioBase:
    subprotocol: str | None
    connection_type: ConnectionType
    receive_bytes: int
    ping_interval: float | None
    max_message_size: int | None
    pong_received: bool
    input_buffer: list[bytes | str]
    incoming_message: bytes | str | None
    incoming_message_len: int
    connected: bool
    is_server: bool
    close_reason: CloseReason
    close_message: str
    rsock: asyncio.StreamReader
    wsock: asyncio.StreamWriter
    event: asyncio.Event
    ws: WSConnection | None
    task: asyncio.Task[None]
    def __init__(
        self,
        connection_type: ConnectionType | None = None,
        receive_bytes: int = 4096,
        ping_interval: float | None = None,
        max_message_size: int | None = None,
    ) -> None: ...
    async def connect(self) -> None: ...
    async def handshake(self) -> None: ...
    # data can be antyhing. a special case is made for `bytes`, anything else is converted to `str`.
    async def send(self, data: bytes | Any) -> None: ...
    async def receive(self, timeout: float | None = None) -> bytes | str | None: ...
    async def close(self, reason: CloseReason | None = None, message: str | None = None) -> None: ...
    def choose_subprotocol(self, request: Request) -> str | None: ...

@type_check_only
class _AioServerRequest(TypedDict):
    # this is `aiohttp.web.Request`
    aiohttp: Incomplete
    sock: None
    headers: None

class AioServer(AioBase):
    request: _AioServerRequest
    headers: dict[str, Any]
    subprotocols: list[str]
    is_server: Literal[True]
    mode: str
    connected: bool
    def __init__(
        self,
        request: _AioServerRequest,
        subprotocols: list[str] | None = None,
        receive_bytes: int = 4096,
        ping_interval: float | None = None,
        max_message_size: int | None = None,
    ) -> None: ...
    @classmethod
    async def accept(
        cls,
        # this is `aiohttp.web.Request`
        aiohttp=None,
        asgi: (
            tuple[
                WSGIEnvironment,
                Callable[[], Awaitable[_SocketDataBytes | _SocketDataStr]],
                Callable[[_SocketDataBase | _SocketDataProtocol | _SocketDataBytes | _SocketDataStr], Awaitable[None]],
            ]
            | None
        ) = None,
        sock: socket.socket | None = None,
        headers: dict[str, Any] | None = None,
        subprotocols: list[str] | None = None,
        receive_bytes: int = 4096,
        ping_interval: float | None = None,
        max_message_size: int | None = None,
    ) -> WebSocketASGI | AioServer: ...
    async def handshake(self) -> None: ...
    def choose_subprotocol(self, request: Request) -> str | None: ...

class AioClient(AioBase):
    url: str
    ssl_context: SSLContext | None
    is_secure: bool
    host: str
    port: int
    path: str
    subprotocols: list[str]
    extra_headeers: list[tuple[bytes, bytes]]
    subprotocol: str | None
    connected: bool
    def __init__(
        self,
        url: str,
        subprotocols: list[str] | None = None,
        headers: dict[str, Any] | None = None,
        receive_bytes: int = 4096,
        ping_interval: float | None = None,
        max_message_size: int | None = None,
        ssl_context: SSLContext | None = None,
    ) -> None: ...
    # the source code itself has this override
    @classmethod
    async def connect(  # type: ignore[override]
        cls,
        url: str,
        subprotocols: list[str] | None = None,
        headers: dict[str, Any] | None = None,
        receive_bytes: int = 4096,
        ping_interval: float | None = None,
        max_message_size: int | None = None,
        ssl_context: SSLContext | None = None,
        thread_class: Unused = None,
        event_class: Unused = None,
    ) -> AioClient: ...
    async def handshake(self) -> None: ...
    async def close(self, reason: CloseReason | None = None, message: str | None = None) -> None: ...



from _typeshed import Incomplete
from collections.abc import Generator
from typing import NamedTuple

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["vf2pp_isomorphism", "vf2pp_is_isomorphic", "vf2pp_all_isomorphisms"]

class _GraphParameters(NamedTuple):
    G1: Incomplete
    G2: Incomplete
    G1_labels: Incomplete
    G2_labels: Incomplete
    nodes_of_G1Labels: Incomplete
    nodes_of_G2Labels: Incomplete
    G2_nodes_of_degree: Incomplete

class _StateParameters(NamedTuple):
    mapping: Incomplete
    reverse_mapping: Incomplete
    T1: Incomplete
    T1_in: Incomplete
    T1_tilde: Incomplete
    T1_tilde_in: Incomplete
    T2: Incomplete
    T2_in: Incomplete
    T2_tilde: Incomplete
    T2_tilde_in: Incomplete

@_dispatchable
def vf2pp_isomorphism(G1: Graph[_Node], G2: Graph[_Node], node_label: str | None = None, default_label: float | None = None): ...
@_dispatchable
def vf2pp_is_isomorphic(
    G1: Graph[_Node], G2: Graph[_Node], node_label: str | None = None, default_label: float | None = None
): ...
@_dispatchable
def vf2pp_all_isomorphisms(
    G1: Graph[_Node], G2: Graph[_Node], node_label: str | None = None, default_label: float | None = None
) -> Generator[Incomplete, None, Incomplete]: ...



import argparse
from _typeshed import Incomplete
from collections.abc import Callable, Sequence
from enum import Enum
from logging import Logger
from typing import Any

from ..plugins.finder import Plugins

LOG: Logger

class _ARG(Enum):
    NO = 1

class Option:
    short_option_name: Incomplete
    long_option_name: Incomplete
    option_args: Incomplete
    action: Incomplete
    default: Incomplete
    type: Incomplete
    dest: Incomplete
    nargs: Incomplete
    const: Incomplete
    choices: Incomplete
    help: Incomplete
    metavar: Incomplete
    required: Incomplete
    option_kwargs: Incomplete
    parse_from_config: Incomplete
    comma_separated_list: Incomplete
    normalize_paths: Incomplete
    config_name: Incomplete
    def __init__(
        self,
        short_option_name: str | _ARG = ...,
        long_option_name: str | _ARG = ...,
        action: str | type[argparse.Action] | _ARG = ...,
        default: Any | _ARG = ...,
        type: Callable[..., Any] | _ARG = ...,
        dest: str | _ARG = ...,
        nargs: int | str | _ARG = ...,
        const: Any | _ARG = ...,
        choices: Sequence[Any] | _ARG = ...,
        help: str | _ARG = ...,
        metavar: str | _ARG = ...,
        required: bool | _ARG = ...,
        parse_from_config: bool = False,
        comma_separated_list: bool = False,
        normalize_paths: bool = False,
    ) -> None: ...
    @property
    def filtered_option_kwargs(self) -> dict[str, Any]: ...
    def normalize(self, value: Any, *normalize_args: str) -> Any: ...
    def to_argparse(self) -> tuple[list[str], dict[str, Any]]: ...

class OptionManager:
    formatter_names: Incomplete
    parser: Incomplete
    config_options_dict: Incomplete
    options: Incomplete
    extended_default_ignore: Incomplete
    extended_default_select: Incomplete
    def __init__(
        self, *, version: str, plugin_versions: str, parents: list[argparse.ArgumentParser], formatter_names: list[str]
    ) -> None: ...
    def register_plugins(self, plugins: Plugins) -> None: ...
    def add_option(self, *args: Any, **kwargs: Any) -> None: ...
    def extend_default_ignore(self, error_codes: Sequence[str]) -> None: ...
    def extend_default_select(self, error_codes: Sequence[str]) -> None: ...
    def parse_args(self, args: Sequence[str] | None = None, values: argparse.Namespace | None = None) -> argparse.Namespace: ...



from collections.abc import Iterable
from logging import Logger

from oauthlib.common import Request

from ..request_validator import RequestValidator
from ..tokens import TokenBase
from .base import GrantTypeBase, _AuthValidator, _TokenValidator

log: Logger

class RefreshTokenGrant(GrantTypeBase):
    def __init__(
        self,
        request_validator: RequestValidator | None = None,
        issue_new_refresh_tokens: bool = True,
        *,
        post_auth: Iterable[_AuthValidator] | None = None,
        post_token: Iterable[_TokenValidator] | None = None,
        pre_auth: Iterable[_AuthValidator] | None = None,
        pre_token: Iterable[_TokenValidator] | None = None,
        **kwargs,
    ) -> None: ...
    def create_token_response(self, request: Request, token_handler: TokenBase) -> tuple[dict[str, str], str, int | None]: ...
    def validate_token_request(self, request: Request) -> None: ...



from _typeshed import Incomplete
from collections.abc import Iterable
from typing import ClassVar, Final

from authlib.jose.rfc7516 import JWEAlgorithm

class DirectAlgorithm(JWEAlgorithm):
    name: str
    description: str
    def prepare_key(self, raw_data): ...
    def generate_preset(self, enc_alg, key): ...
    def wrap(self, enc_alg, headers, key, preset=None): ...
    def unwrap(self, enc_alg, ek, headers, key): ...

class RSAAlgorithm(JWEAlgorithm):
    key_size: int
    name: str
    description: str
    padding: Incomplete
    def __init__(self, name, description, pad_fn) -> None: ...
    def prepare_key(self, raw_data): ...
    def generate_preset(self, enc_alg, key): ...
    def wrap(self, enc_alg, headers, key, preset=None): ...
    def unwrap(self, enc_alg, ek, headers, key): ...

class AESAlgorithm(JWEAlgorithm):
    name: str
    description: str
    key_size: Incomplete
    def __init__(self, key_size) -> None: ...
    def prepare_key(self, raw_data): ...
    def generate_preset(self, enc_alg, key): ...
    def wrap_cek(self, cek, key): ...
    def wrap(self, enc_alg, headers, key, preset=None): ...
    def unwrap(self, enc_alg, ek, headers, key): ...

class AESGCMAlgorithm(JWEAlgorithm):
    EXTRA_HEADERS: ClassVar[Iterable[str]]
    name: str
    description: str
    key_size: Incomplete
    def __init__(self, key_size) -> None: ...
    def prepare_key(self, raw_data): ...
    def generate_preset(self, enc_alg, key): ...
    def wrap(self, enc_alg, headers, key, preset=None): ...
    def unwrap(self, enc_alg, ek, headers, key): ...

class ECDHESAlgorithm(JWEAlgorithm):
    EXTRA_HEADERS: ClassVar[Iterable[str]]
    ALLOWED_KEY_CLS = Incomplete
    name: str
    description: str
    key_size: Incomplete
    aeskw: Incomplete
    def __init__(self, key_size=None) -> None: ...
    def prepare_key(self, raw_data): ...
    def generate_preset(self, enc_alg, key): ...
    def compute_fixed_info(self, headers, bit_size): ...
    def compute_derived_key(self, shared_key, fixed_info, bit_size): ...
    def deliver(self, key, pubkey, headers, bit_size): ...
    def wrap(self, enc_alg, headers, key, preset=None): ...
    def unwrap(self, enc_alg, ek, headers, key): ...

def u32be_len_input(s, base64: bool = False): ...

JWE_ALG_ALGORITHMS: Final[list[JWEAlgorithm]]



import logging
from typing import TextIO

__all__ = ["Debug", "setLogger", "hexdump"]

class Printer:
    def __init__(
        self,
        logger: logging.Logger | None = None,
        handler: logging.StreamHandler[TextIO] | None = None,
        formatter: logging.Formatter | None = None,
    ) -> None: ...
    def __call__(self, msg) -> None: ...

class Debug:
    defaultPrinter: Printer
    def __init__(self, *flags, **options) -> None: ...
    def __call__(self, msg) -> None: ...
    def __and__(self, flag): ...
    def __rand__(self, flag): ...

def setLogger(userLogger) -> None: ...
def hexdump(octets): ...

class Scope:
    def __init__(self) -> None: ...
    def push(self, token) -> None: ...
    def pop(self): ...



from typing import Literal, TypedDict, type_check_only
from typing_extensions import NotRequired, TypeAlias

_NodeType: TypeAlias = Literal[
    "comparator",
    "current",
    "expref",
    "function_expression",
    "field",
    "filter_projection",
    "flatten",
    "identity",
    "index",
    "index_expression",
    "key_val_pair",
    "literal",
    "multi_select_dict",
    "multi_select_list",
    "or_expression",
    "and_expression",
    "not_expression",
    "pipe",
    "projection",
    "subexpression",
    "slice",
    "value_projection",
]

@type_check_only
class _ASTNode(TypedDict):
    type: _NodeType
    children: list[_ASTNode]
    value: NotRequired[str]

def comparator(name: str, first: _ASTNode, second: _ASTNode) -> _ASTNode: ...
def current_node() -> _ASTNode: ...
def expref(expression: _ASTNode) -> _ASTNode: ...
def function_expression(name: str, args: list[_ASTNode]) -> _ASTNode: ...
def field(name: str) -> _ASTNode: ...
def filter_projection(left: _ASTNode, right: _ASTNode, comparator: _ASTNode) -> _ASTNode: ...
def flatten(node: _ASTNode) -> _ASTNode: ...
def identity() -> _ASTNode: ...
def index(index: str) -> _ASTNode: ...
def index_expression(children: list[_ASTNode]) -> _ASTNode: ...
def key_val_pair(key_name: str, node: _ASTNode) -> _ASTNode: ...
def literal(literal_value: str) -> _ASTNode: ...
def multi_select_dict(nodes: list[_ASTNode]) -> _ASTNode: ...
def multi_select_list(nodes: list[_ASTNode]) -> _ASTNode: ...
def or_expression(left: _ASTNode, right: _ASTNode) -> _ASTNode: ...
def and_expression(left: _ASTNode, right: _ASTNode) -> _ASTNode: ...
def not_expression(expr: _ASTNode) -> _ASTNode: ...
def pipe(left: _ASTNode, right: _ASTNode) -> _ASTNode: ...
def projection(left: _ASTNode, right: _ASTNode) -> _ASTNode: ...
def subexpression(children: list[_ASTNode]) -> _ASTNode: ...
def slice(start: _ASTNode, end: _ASTNode, step: _ASTNode) -> _ASTNode: ...
def value_projection(left: _ASTNode, right: _ASTNode) -> _ASTNode: ...



from logging import Logger

from .base import BaseEndpoint as BaseEndpoint

log: Logger

class SignatureOnlyEndpoint(BaseEndpoint):
    def validate_request(self, uri, http_method: str = "GET", body=None, headers=None): ...



from _typeshed import Incomplete

from braintree.address import Address
from braintree.braintree_gateway import BraintreeGateway
from braintree.error_result import ErrorResult
from braintree.successful_result import SuccessfulResult

class AddressGateway:
    gateway: BraintreeGateway
    config: Incomplete
    def __init__(self, gateway: BraintreeGateway) -> None: ...
    def create(self, params: dict[str, Incomplete] | None = None) -> SuccessfulResult | ErrorResult | None: ...
    def delete(self, customer_id: str, address_id: str) -> SuccessfulResult: ...
    def find(self, customer_id: str, address_id: str) -> Address: ...
    def update(
        self, customer_id: str, address_id: str, params: dict[str, Incomplete] | None = None
    ) -> SuccessfulResult | ErrorResult | None: ...



from _typeshed import Incomplete
from collections.abc import Iterable, Mapping
from typing import Any, Final, Literal

from docker._types import ContainerWeightDevice

from .. import errors
from .base import DictType
from .healthcheck import Healthcheck
from .networks import NetworkingConfig
from .services import Mount

class LogConfigTypesEnum:
    JSON: Final = "json-file"
    SYSLOG: Final = "syslog"
    JOURNALD: Final = "journald"
    GELF: Final = "gelf"
    FLUENTD: Final = "fluentd"
    NONE: Final = "none"

class LogConfig(DictType):
    types: type[LogConfigTypesEnum]
    def __init__(self, **kwargs) -> None: ...
    @property
    def type(self): ...
    @type.setter
    def type(self, value) -> None: ...
    @property
    def config(self): ...
    def set_config_value(self, key, value) -> None: ...
    def unset_config(self, key) -> None: ...

class Ulimit(DictType):
    def __init__(self, **kwargs) -> None: ...
    @property
    def name(self): ...
    @name.setter
    def name(self, value) -> None: ...
    @property
    def soft(self): ...
    @soft.setter
    def soft(self, value) -> None: ...
    @property
    def hard(self): ...
    @hard.setter
    def hard(self, value) -> None: ...

class DeviceRequest(DictType):
    def __init__(self, **kwargs) -> None: ...
    @property
    def driver(self): ...
    @driver.setter
    def driver(self, value) -> None: ...
    @property
    def count(self): ...
    @count.setter
    def count(self, value) -> None: ...
    @property
    def device_ids(self): ...
    @device_ids.setter
    def device_ids(self, value) -> None: ...
    @property
    def capabilities(self): ...
    @capabilities.setter
    def capabilities(self, value) -> None: ...
    @property
    def options(self): ...
    @options.setter
    def options(self, value) -> None: ...

class HostConfig(dict[str, Incomplete]):
    def __init__(
        self,
        version: str,
        binds: dict[str, Mapping[str, str]] | list[str] | None = None,
        port_bindings: Mapping[int | str, Incomplete] | None = None,
        lxc_conf: dict[str, Incomplete] | list[dict[str, Incomplete]] | None = None,
        publish_all_ports: bool = False,
        links: dict[str, str] | dict[str, None] | dict[str, str | None] | Iterable[tuple[str, str | None]] | None = None,
        privileged: bool = False,
        dns: list[str] | None = None,
        dns_search: list[str] | None = None,
        volumes_from: list[str] | None = None,
        network_mode: str | None = None,
        restart_policy: Mapping[str, str | int] | None = None,
        cap_add: list[str] | None = None,
        cap_drop: list[str] | None = None,
        devices: list[str] | None = None,
        extra_hosts: dict[str, Incomplete] | list[Incomplete] | None = None,
        read_only: bool | None = None,
        pid_mode: str | None = None,
        ipc_mode: str | None = None,
        security_opt: list[str] | None = None,
        ulimits: list[Ulimit] | None = None,
        log_config: LogConfig | None = None,
        mem_limit: str | int | None = None,
        memswap_limit: str | int | None = None,
        mem_reservation: str | int | None = None,
        kernel_memory: str | int | None = None,
        mem_swappiness: int | None = None,
        cgroup_parent: str | None = None,
        group_add: Iterable[str | int] | None = None,
        cpu_quota: int | None = None,
        cpu_period: int | None = None,
        blkio_weight: int | None = None,
        blkio_weight_device: list[ContainerWeightDevice] | None = None,
        device_read_bps: list[Mapping[str, str | int]] | None = None,
        device_write_bps: list[Mapping[str, str | int]] | None = None,
        device_read_iops: list[Mapping[str, str | int]] | None = None,
        device_write_iops: list[Mapping[str, str | int]] | None = None,
        oom_kill_disable: bool = False,
        shm_size: str | int | None = None,
        sysctls: dict[str, str] | None = None,
        tmpfs: dict[str, str] | None = None,
        oom_score_adj: int | None = None,
        dns_opt: list[Incomplete] | None = None,
        cpu_shares: int | None = None,
        cpuset_cpus: str | None = None,
        userns_mode: str | None = None,
        uts_mode: str | None = None,
        pids_limit: int | None = None,
        isolation: str | None = None,
        auto_remove: bool = False,
        storage_opt: dict[Incomplete, Incomplete] | None = None,
        init: bool | None = None,
        init_path: str | None = None,
        volume_driver: str | None = None,
        cpu_count: int | None = None,
        cpu_percent: int | None = None,
        nano_cpus: int | None = None,
        cpuset_mems: str | None = None,
        runtime: str | None = None,
        mounts: list[Mount] | None = None,
        cpu_rt_period: int | None = None,
        cpu_rt_runtime: int | None = None,
        device_cgroup_rules: list[Incomplete] | None = None,
        device_requests: list[DeviceRequest] | None = None,
        cgroupns: Literal["private", "host"] | None = None,
    ) -> None: ...

def host_config_type_error(param: str, param_value: object, expected: str) -> TypeError: ...
def host_config_version_error(param: str, version: str, less_than: bool = True) -> errors.InvalidVersion: ...
def host_config_value_error(param: str, param_value: object) -> ValueError: ...
def host_config_incompatible_error(param: str, param_value: str, incompatible_param: str) -> errors.InvalidArgument: ...

class ContainerConfig(dict[str, Incomplete]):
    def __init__(
        self,
        version: str,
        image: str,
        command: str | list[str],
        hostname: str | None = None,
        user: str | int | None = None,
        detach: bool = False,
        stdin_open: bool = False,
        tty: bool = False,
        # list is invariant, enumerating all possible union combination would be too complex for:
        # list[str | int | tuple[int | str, str] | tuple[int | str, ...]]
        ports: dict[str, dict[Incomplete, Incomplete]] | list[Any] | None = None,
        environment: dict[str, str] | list[str] | None = None,
        volumes: str | list[str] | None = None,
        network_disabled: bool = False,
        entrypoint: str | list[str] | None = None,
        working_dir: str | None = None,
        domainname: str | None = None,
        host_config: HostConfig | None = None,
        mac_address: str | None = None,
        labels: dict[str, str] | list[str] | None = None,
        stop_signal: str | None = None,
        networking_config: NetworkingConfig | None = None,
        healthcheck: Healthcheck | None = None,
        stop_timeout: int | None = None,
        runtime: str | None = None,
    ) -> None: ...



from typing import Any

from yaml.error import MarkedYAMLError
from yaml.nodes import MappingNode, Node, ScalarNode, SequenceNode

class ComposerError(MarkedYAMLError): ...

class Composer:
    anchors: dict[Any, Node]
    def __init__(self) -> None: ...
    def check_node(self) -> bool: ...
    def get_node(self) -> Node | None: ...
    def get_single_node(self) -> Node | None: ...
    def compose_document(self) -> Node | None: ...
    def compose_node(self, parent: Node | None, index: int) -> Node | None: ...
    def compose_scalar_node(self, anchor: dict[Any, Node]) -> ScalarNode: ...
    def compose_sequence_node(self, anchor: dict[Any, Node]) -> SequenceNode: ...
    def compose_mapping_node(self, anchor: dict[Any, Node]) -> MappingNode: ...

__all__ = ["Composer", "ComposerError"]



from collections import OrderedDict
from collections.abc import Generator
from typing import Final

from .dates import BaseDate, HebrewDate

PARSHIOS: Final[list[str]]
PARSHIOS_HEBREW: Final[list[str]]

def getparsha(date: BaseDate, israel: bool = False) -> list[int] | None: ...
def getparsha_string(date: BaseDate, israel: bool = False, hebrew: bool = False) -> str | None: ...
def iterparshios(year: int, israel: bool = False) -> Generator[list[int] | None]: ...
def parshatable(year: int, israel: bool = False) -> OrderedDict[HebrewDate, list[int] | None]: ...
def four_parshios(date: BaseDate, hebrew: bool = False) -> str: ...



class ProgressCalculator:
    SAMPLING_WINDOW: int
    SAMPLING_RATE: float
    GRACE_PERIOD: int
    downloaded: int
    elapsed: float
    speed: SmoothValue
    eta: SmoothValue
    def __init__(self, initial: int) -> None: ...
    @property
    def total(self) -> int | None: ...
    @total.setter
    def total(self, value: int | None) -> None: ...
    def thread_reset(self) -> None: ...
    def update(self, size: int | None) -> None: ...

class SmoothValue:
    value: float | None
    def __init__(self, initial: float | None, smoothing: float) -> None: ...
    smooth: float | None
    def set(self, value: float) -> None: ...
    def reset(self) -> None: ...



from _typeshed import Incomplete
from collections.abc import Iterable

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable
from numpy.random import RandomState

__all__ = ["maximal_independent_set"]

@_dispatchable
def maximal_independent_set(
    G: Graph[_Node], nodes: Iterable[Incomplete] | None = None, seed: int | RandomState | None = None
) -> list[Incomplete]: ...



from _typeshed import Incomplete

from pyasn1.type.base import Asn1Type

class ExtendedOperation:
    connection: Incomplete
    decoded_response: Incomplete | None
    result: Incomplete | None
    asn1_spec: Asn1Type | None
    request_name: Incomplete | None
    response_name: Incomplete | None
    request_value: Asn1Type | None
    response_value: Incomplete | None
    response_attribute: Incomplete | None
    controls: Incomplete
    def __init__(self, connection, controls=None) -> None: ...
    def send(self): ...
    def populate_result(self) -> None: ...
    def decode_response(self, response=None) -> None: ...
    def set_response(self) -> None: ...
    def config(self) -> None: ...



from nanoleafapi.digital_twin import NanoleafDigitalTwin as NanoleafDigitalTwin
from nanoleafapi.nanoleaf import (
    BLUE as BLUE,
    GREEN as GREEN,
    LIGHT_BLUE as LIGHT_BLUE,
    ORANGE as ORANGE,
    PINK as PINK,
    PURPLE as PURPLE,
    RED as RED,
    WHITE as WHITE,
    YELLOW as YELLOW,
    Nanoleaf as Nanoleaf,
    NanoleafConnectionError as NanoleafConnectionError,
    NanoleafEffectCreationError as NanoleafEffectCreationError,
    NanoleafRegistrationError as NanoleafRegistrationError,
)



import sys
from typing import Any, ClassVar, Final

import click
from click_web.web_click_types import EmailParamType, PasswordParamType, TextAreaParamType

class FieldId:
    SEPARATOR: ClassVar[str]
    command_index: int
    param_index: int
    param_type: str
    click_type: str
    nargs: int
    form_type: str
    name: str
    key: str

    def __init__(
        self,
        command_index: int,
        param_index: int,
        param_type: str,
        click_type: str,
        nargs: int,
        form_type: str,
        name: str,
        key: str | None = None,
    ) -> None: ...
    @classmethod
    def from_string(cls, field_info_as_string: str) -> FieldId: ...

class NotSupported(ValueError): ...

class BaseInput:
    param_type_cls: type[click.types.ParamType] | None
    ctx: click.Context
    param: click.Parameter
    command_index: int
    param_index: int
    def __init__(self, ctx: click.Context, param: click.Parameter, command_index: int, param_index: int) -> None: ...
    def is_supported(self) -> bool: ...
    @property
    def fields(self) -> dict[str, Any]: ...
    @property
    def type_attrs(self) -> dict[str, Any]: ...
    def _to_cmd_line_name(self, name: str) -> str: ...
    def _build_name(self, name: str) -> str: ...

class ChoiceInput(BaseInput):
    if sys.version_info >= (3, 10):
        param_type_cls: type[click.Choice[Any]]
    else:
        param_type_cls: type[click.Choice]

class FlagInput(BaseInput):
    param_type_cls: None

class IntInput(BaseInput):
    param_type_cls: type[click.types.IntParamType]

class FloatInput(BaseInput):
    param_type_cls: type[click.types.FloatParamType]

class FolderInput(BaseInput):
    param_type_cls: None

class FileInput(BaseInput):
    param_type_cls: None

class EmailInput(BaseInput):
    param_type_cls: type[EmailParamType]

class PasswordInput(BaseInput):
    param_type_cls: type[PasswordParamType]

class TextAreaInput(BaseInput):
    param_type_cls: type[TextAreaParamType]

class DefaultInput(BaseInput):
    param_type_cls: type[click.ParamType]

INPUT_TYPES: Final[list[type[BaseInput]]]
_DEFAULT_INPUT: Final[list[type[DefaultInput]]]

def get_input_field(ctx: click.Context, param: click.Parameter, command_index: int, param_index: int) -> dict[str, Any]: ...



import json
from typing import Any

from collections import namedtuple

from lsprotocol import converters

from pygls.protocol.json_rpc import (
    JsonRPCNotification,
    JsonRPCProtocol,
    JsonRPCRequestMessage,
    JsonRPCResponseMessage,
)
from pygls.protocol.language_server import LanguageServerProtocol, lsp_method
from pygls.protocol.lsp_meta import LSPMeta, call_user_feature


def _dict_to_object(d: Any):
    """Create nested objects (namedtuple) from dict."""

    if d is None:
        return None

    if not isinstance(d, dict):
        return d

    type_name = d.pop("type_name", "Object")
    return json.loads(
        json.dumps(d),
        object_hook=lambda p: namedtuple(type_name, p.keys(), rename=True)(*p.values()),
    )


def _params_field_structure_hook(obj, cls):
    if "params" in obj:
        obj["params"] = _dict_to_object(obj["params"])

    return cls(**obj)


def _result_field_structure_hook(obj, cls):
    if "result" in obj:
        obj["result"] = _dict_to_object(obj["result"])

    return cls(**obj)


def default_converter():
    """Default converter factory function."""

    converter = converters.get_converter()
    converter.register_structure_hook(
        JsonRPCRequestMessage, _params_field_structure_hook
    )

    converter.register_structure_hook(
        JsonRPCResponseMessage, _result_field_structure_hook
    )

    converter.register_structure_hook(JsonRPCNotification, _params_field_structure_hook)

    return converter


__all__ = (
    "JsonRPCProtocol",
    "LanguageServerProtocol",
    "JsonRPCRequestMessage",
    "JsonRPCResponseMessage",
    "JsonRPCNotification",
    "LSPMeta",
    "call_user_feature",
    "_dict_to_object",
    "_params_field_structure_hook",
    "_result_field_structure_hook",
    "default_converter",
    "lsp_method",
)



from .resource import Collection, Model

class Node(Model):
    id_attribute: str
    @property
    def version(self): ...
    def update(self, node_spec): ...
    def remove(self, force: bool = False): ...

class NodeCollection(Collection[Node]):
    model: type[Node]
    def get(self, node_id): ...
    def list(self, *args, **kwargs): ...



from collections.abc import Collection, Iterator, KeysView
from enum import Enum
from http.cookiejar import Cookie, CookiePolicy, MozillaCookieJar
from http.cookies import SimpleCookie
from typing import Final, TextIO, TypeVar

from . import _LoggerProtocol
from .minicurses import MultilinePrinter
from .utils._utils import YoutubeDLError
from .YoutubeDL import YoutubeDL

CHROMIUM_BASED_BROWSERS: Final[set[str]]
SUPPORTED_BROWSERS: Final[set[str]]

class _LinuxKeyring(Enum):
    BASICTEXT = 5
    GNOMEKEYRING = 4
    KWALLET = 1
    KWALLET5 = 2
    KWALLET6 = 3

SUPPORTED_KEYRINGS: Final[KeysView[str]]

class YDLLogger(_LoggerProtocol):
    def warning(self, message: str, only_once: bool = False) -> None: ...  # type: ignore[override]

    class ProgressBar(MultilinePrinter):
        def print(self, message: str) -> None: ...

    def progress_bar(self) -> ProgressBar: ...

class CookieLoadError(YoutubeDLError): ...

class YoutubeDLCookieJar(MozillaCookieJar):
    def __init__(self, filename: str | None = None, delayload: bool = False, policy: CookiePolicy | None = None) -> None: ...
    def open(self, file: str, *, write: bool = False) -> Iterator[TextIO]: ...
    def get_cookie_header(self, url: str) -> str: ...
    def get_cookies_for_url(self, url: str) -> list[Cookie]: ...
    def load(self, filename: str | None = None, ignore_discard: bool = True, ignore_expires: bool = True) -> None: ...
    def save(self, filename: str | None = None, ignore_discard: bool = True, ignore_expires: bool = True) -> None: ...

def load_cookies(cookie_file: str, browser_specification: str | None, ydl: YoutubeDL) -> YoutubeDLCookieJar: ...
def extract_cookies_from_browser(
    browser_name: str,
    profile: str | None = None,
    logger: _LoggerProtocol = ...,
    *,
    keyring: _LinuxKeyring | None = None,
    container: str | None = None,
) -> YoutubeDLCookieJar: ...

_T = TypeVar("_T", bound=MozillaCookieJar)

def parse_safari_cookies(data: bytes, jar: _T | None = None, logger: _LoggerProtocol = ...) -> _T: ...

class ChromeCookieDecryptor:
    def decrypt(self, encrypted_value: bytes) -> str: ...

class LinuxChromeCookieDecryptor(ChromeCookieDecryptor):
    def __init__(
        self,
        browser_keyring_name: str,
        logger: _LoggerProtocol,
        *,
        keyring: _LinuxKeyring | None = None,
        meta_version: int | None = None,
    ) -> None: ...
    @staticmethod
    def derive_key(password: bytes) -> bytes: ...

class MacChromeCookieDecryptor(ChromeCookieDecryptor):
    def __init__(self, browser_keyring_name: str, logger: YDLLogger, meta_version: int | None = None) -> None: ...
    @staticmethod
    def derive_key(password: bytes) -> bytes: ...

class WindowsChromeCookieDecryptor(ChromeCookieDecryptor):
    def __init__(self, browser_root: str, logger: YDLLogger, meta_version: int | None = None) -> None: ...

def get_cookie_decryptor(
    browser_root: str,
    browser_keyring_name: str,
    logger: _LoggerProtocol,
    *,
    keyring: _LinuxKeyring | None = None,
    meta_version: int | None = None,
) -> ChromeCookieDecryptor: ...

class ParserError(Exception): ...

class DataParser:
    def __init__(self, data: bytes, logger: YDLLogger) -> None: ...
    def read_bytes(self, num_bytes: int) -> bytes: ...
    def expect_bytes(self, expected_value: bytes, message: str) -> None: ...
    def read_uint(self, big_endian: bool = False) -> int: ...
    def read_double(self, big_endian: bool = False) -> float: ...
    def read_cstring(self) -> bytes: ...
    def skip(self, num_bytes: int, description: str = "unknown") -> None: ...
    def skip_to(self, offset: int, description: str = "unknown") -> None: ...
    def skip_to_end(self, description: str = "unknown") -> None: ...

def pbkdf2_sha1(password: bytes, salt: bytes, iterations: int, key_length: int) -> bytes: ...

class LenientSimpleCookie(SimpleCookie):
    def load(self, data: str | Collection[str]) -> None: ...



#!/usr/bin/env python3
import sys

from ts_utils.requirements import get_stubtest_system_requirements

if __name__ == "__main__":
    distributions = sys.argv[1:]
    for requirement in sorted(get_stubtest_system_requirements(distributions)):
        print(requirement)



from typing import TypedDict, type_check_only

@type_check_only
class _SupportedWorkers(TypedDict):
    sync: str
    eventlet: str
    gevent: str
    gevent_wsgi: str
    gevent_pywsgi: str
    tornado: str
    gthread: str

SUPPORTED_WORKERS: _SupportedWorkers



from ._main import *

# Sync with regex._main.__all__
__all__ = [
    "cache_all",
    "compile",
    "DEFAULT_VERSION",
    "escape",
    "findall",
    "finditer",
    "fullmatch",
    "match",
    "purge",
    "search",
    "split",
    "splititer",
    "sub",
    "subf",
    "subfn",
    "subn",
    "template",
    "Scanner",
    "A",
    "ASCII",
    "B",
    "BESTMATCH",
    "D",
    "DEBUG",
    "E",
    "ENHANCEMATCH",
    "S",
    "DOTALL",
    "F",
    "FULLCASE",
    "I",
    "IGNORECASE",
    "L",
    "LOCALE",
    "M",
    "MULTILINE",
    "P",
    "POSIX",
    "R",
    "REVERSE",
    "T",
    "TEMPLATE",
    "U",
    "UNICODE",
    "V0",
    "VERSION0",
    "V1",
    "VERSION1",
    "X",
    "VERBOSE",
    "W",
    "WORD",
    "error",
    "Regex",
    "__version__",
    "__doc__",
    "RegexFlag",
    "Pattern",
    "Match",
]



from collections.abc import Sequence

from Xlib._typing import ErrorHandler
from Xlib.protocol import request
from Xlib.protocol.structs import _RGB3IntIterable
from Xlib.xobject import cursor, resource

class Fontable(resource.Resource):
    __fontable__ = resource.Resource.__resource__
    def query(self) -> request.QueryFont: ...
    def query_text_extents(self, string: str) -> request.QueryTextExtents: ...

class GC(Fontable):
    __gc__ = resource.Resource.__resource__
    def change(self, onerror: ErrorHandler[object] | None = None, **keys: object) -> None: ...
    def copy(self, src_gc: int, mask: int, onerror: ErrorHandler[object] | None = None) -> None: ...
    def set_dashes(self, offset: int, dashes: Sequence[int], onerror: ErrorHandler[object] | None = None) -> None: ...
    def set_clip_rectangles(
        self,
        x_origin: int,
        y_origin: int,
        rectangles: Sequence[dict[str, int]],
        ordering: int,
        onerror: ErrorHandler[object] | None = None,
    ) -> None: ...
    def free(self, onerror: ErrorHandler[object] | None = None) -> None: ...

class Font(Fontable):
    __font__ = resource.Resource.__resource__
    def close(self, onerror: ErrorHandler[object] | None = None) -> None: ...
    def create_glyph_cursor(
        self, mask: Font, source_char: int, mask_char: int, foreground: _RGB3IntIterable, background: _RGB3IntIterable
    ) -> cursor.Cursor: ...



from collections.abc import Iterator, MutableSet
from re import Pattern
from typing import Any, TypedDict, type_check_only
from typing_extensions import deprecated
from xml.etree.ElementTree import Element

from markdown.core import Markdown
from markdown.extensions import Extension
from markdown.treeprocessors import Treeprocessor

IDCOUNT_RE: Pattern[str]

@type_check_only
class _FlatTocToken(TypedDict):
    level: int
    id: str
    name: str

@type_check_only
class _TocToken(_FlatTocToken):
    children: list[_TocToken]

def slugify(value: str, separator: str, unicode: bool = False) -> str: ...
def slugify_unicode(value: str, separator: str) -> str: ...
def unique(id: str, ids: MutableSet[str]) -> str: ...
@deprecated("Use `render_inner_html` and `striptags` instead.")
def get_name(el: Element) -> str: ...
@deprecated("Use `run_postprocessors`, `render_inner_html` and/or `striptags` instead.")
def stashedHTML2text(text: str, md: Markdown, strip_entities: bool = True) -> str: ...
def unescape(text: str) -> str: ...
def strip_tags(text: str) -> str: ...
def escape_cdata(text: str) -> str: ...
def run_postprocessors(text: str, md: Markdown) -> str: ...
def render_inner_html(el: Element, md: Markdown) -> str: ...
def remove_fnrefs(root: Element) -> Element: ...
def nest_toc_tokens(toc_list: list[_FlatTocToken]) -> list[_TocToken]: ...

class TocTreeprocessor(Treeprocessor):
    marker: str
    title: str
    base_level: int
    slugify: Any
    sep: Any
    toc_class: Any
    title_class: str
    use_anchors: bool
    anchorlink_class: str
    use_permalinks: bool
    permalink_class: str
    permalink_title: str
    permalink_leading: bool
    header_rgx: Pattern[str]
    toc_top: int
    toc_bottom: int
    def __init__(self, md: Markdown, config: dict[str, Any]) -> None: ...
    def iterparent(self, node: Element) -> Iterator[tuple[Element, Element]]: ...
    def replace_marker(self, root: Element, elem: Element) -> None: ...
    def set_level(self, elem: Element) -> None: ...
    def add_anchor(self, c: Element, elem_id: str) -> None: ...
    def add_permalink(self, c: Element, elem_id: str) -> None: ...
    def build_toc_div(self, toc_list: list[_TocToken]) -> Element: ...
    def run(self, doc: Element) -> None: ...

class TocExtension(Extension):
    TreeProcessorClass: type[TocTreeprocessor]
    def __init__(self, **kwargs) -> None: ...
    md: Markdown
    def reset(self) -> None: ...

def makeExtension(**kwargs) -> TocExtension: ...



import tarfile

with tarfile.open("test.tar.xz", "w:xz") as tar:
    pass

# Test with valid preset values
tarfile.open("test.tar.xz", "w:xz", preset=0)
tarfile.open("test.tar.xz", "w:xz", preset=5)
tarfile.open("test.tar.xz", "w:xz", preset=9)

# Test with invalid preset values
tarfile.open("test.tar.xz", "w:xz", preset=-1)  # type: ignore
tarfile.open("test.tar.xz", "w:xz", preset=10)  # type: ignore

# Test pipe modes
tarfile.open("test.tar.xz", "r|*")
tarfile.open("test.tar.xz", mode="r|*")



import dataclasses
import importlib.abc
from _typeshed import Unused
from collections.abc import Iterator
from importlib.machinery import ModuleSpec
from pathlib import Path
from types import ModuleType
from typing import Any, Final

from .globals import Indirect

__all__ = [
    "COMPAT_PACKAGE_NAME",
    "PACKAGE_NAME",
    "PluginSpec",
    "directories",
    "load_all_plugins",
    "load_plugins",
    "register_plugin_spec",
]

PACKAGE_NAME: Final = "yt_dlp_plugins"
COMPAT_PACKAGE_NAME: Final = "ytdlp_plugins"

@dataclasses.dataclass
class PluginSpec:
    module_name: str
    suffix: str
    destination: Indirect[dict[str, Any]]
    plugin_destination: Indirect[dict[str, Any]]

class PluginLoader(importlib.abc.Loader):
    def exec_module(self, module: ModuleType) -> None: ...

class PluginFinder(importlib.abc.MetaPathFinder):
    packages: set[str]
    def __init__(self, *packages: str) -> None: ...
    def search_locations(self, fullname: str) -> Iterator[Path]: ...
    def find_spec(self, fullname: str, path: Unused | None = None, target: Unused | None = None) -> ModuleSpec | None: ...
    def invalidate_caches(self) -> None: ...

def directories() -> list[str]: ...
def load_plugins(plugin_spec: PluginSpec) -> dict[str, Any]: ...
def load_all_plugins() -> None: ...
def register_plugin_spec(plugin_spec: PluginSpec) -> None: ...



from _typeshed import SupportsWrite
from collections.abc import Callable
from types import FrameType
from typing import Any

from .watch_element import WatchElement

class WatchPrint:
    # User-defined callbacks passed to `__init__` set as instance variables have arguments of type `Any` to be
    # compatible with more precisely-annotated signatures. These callbacks are passed from `watchpoints.watch.Watch`.

    custom_printer: Callable[[Any], None] | None
    file: str | SupportsWrite[str] | None
    stack_limit: int | None

    def __init__(
        self,
        file: str | SupportsWrite[str] | None = ...,
        stack_limit: int | None = None,
        custom_printer: Callable[[Any], None] | None = None,  # User-defined printing callback
    ) -> None: ...
    def __call__(self, frame: FrameType, elem: WatchElement, exec_info: tuple[str, str, int | None]) -> None: ...
    def getsourceline(self, exec_info: tuple[str, str, int | None]) -> str: ...
    def printer(self, obj: object) -> None: ...



from typing import Final, Literal
from typing_extensions import deprecated

MON: Final = 0
TUE: Final = 1
WED: Final = 2
THU: Final = 3
FRI: Final = 4
SAT: Final = 5
SUN: Final = 6
JAN: Final = 1
FEB: Final = 2
MAR: Final = 3
APR: Final = 4
MAY: Final = 5
JUN: Final = 6
JUL: Final = 7
AUG: Final = 8
SEP: Final = 9
OCT: Final = 10
NOV: Final = 11
DEC: Final = 12

def new_years(year: int, observed: bool | None = None) -> tuple[int, int, int]: ...
def martin_luther_king_day(year: int) -> tuple[int, int, int]: ...
def lincolns_birthday(year: int) -> tuple[int, int, int]: ...
def valentines_day(year: int) -> tuple[int, int, int]: ...
def washingtons_birthday(year: int, observed: bool | None = None) -> tuple[int, int, int]: ...
def presidents_day(year: int) -> tuple[int, int, int]: ...
def pulaski_day(year: int) -> tuple[int, int, int]: ...
def easter(year: int, church: Literal["western", "orthodox", "eastern"] | None = None) -> tuple[int, int, int]: ...
def may_day(year: int) -> tuple[int, int, int]: ...
def mothers_day(year: int) -> tuple[int, int, int]: ...
def memorial_day(year: int) -> tuple[int, int, int]: ...
def fathers_day(year: int) -> tuple[int, int, int]: ...
def juneteenth(year: int) -> tuple[int, int, int]: ...
def flag_day(year: int) -> tuple[int, int, int]: ...
def independence_day(year: int, observed: bool | None = None) -> tuple[int, int, int]: ...
def labor_day(year: int) -> tuple[int, int, int]: ...
def indigenous_peoples_day(year: int, country: str = "usa") -> tuple[int, int, int]: ...
@deprecated("The `columbus_day` function will be removed in a future release. Use `indigenous_peoples_day` instead.")
def columbus_day(year: int, country: str = "usa") -> tuple[int, int, int]: ...
def halloween(year: int) -> tuple[int, int, int]: ...
def election_day(year: int) -> tuple[int, int, int]: ...
def veterans_day(year: int, observed: bool | None = None) -> tuple[int, int, int]: ...
def rememberance_day(year: int) -> tuple[int, int, int]: ...
def armistice_day(year: int) -> tuple[int, int, int]: ...
def thanksgiving(year: int, country: Literal["usa", "canada"] = "usa") -> tuple[int, int, int]: ...
def christmas_eve(year: int) -> tuple[int, int, int]: ...
def christmas(year: int, observed: bool | None = None) -> tuple[int, int, int]: ...
def new_years_eve(year: int) -> tuple[int, int, int]: ...
def hanukkah(year: int, eve: bool | None = None) -> tuple[int, int, int]: ...
def purim(year: int, eve: bool | None = None) -> tuple[int, int, int]: ...
def rosh_hashanah(year: int, eve: bool | None = None) -> tuple[int, int, int]: ...
def yom_kippur(year: int, eve: bool | None = None) -> tuple[int, int, int]: ...
def passover(year: int, eve: bool | None = None) -> tuple[int, int, int]: ...
def shavuot(year: int, eve: bool | None = None) -> tuple[int, int, int]: ...
def sukkot(year: int, eve: bool | None = None) -> tuple[int, int, int]: ...
def shemini_azeret(year: int, eve: bool | None = None) -> tuple[int, int, int]: ...
def lag_baomer(year: int, eve: bool | None = None) -> tuple[int, int, int]: ...
def tu_beshvat(year: int, eve: bool | None = None) -> tuple[int, int, int]: ...
def tisha_bav(year: int, eve: bool | None = None) -> tuple[int, int, int]: ...
def dia_constitucion(year: int, observed: bool | None = True) -> tuple[int, int, int]: ...
def natalicio_benito_juarez(year: int, observed: bool | None = True) -> tuple[int, int, int]: ...
def dia_independencia(year: int) -> tuple[int, int, int]: ...
def dia_revolucion(year: int) -> tuple[int, int, int]: ...
def ramadan(year: int, eve: bool | None = None) -> tuple[int, int, int]: ...
def ashura(year: int, eve: bool | None = None) -> tuple[int, int, int]: ...
def eid_alfitr(year: int, eve: bool | None = None) -> tuple[int, int, int]: ...
def eid_aladha(year: int, eve: bool | None = None) -> tuple[int, int, int]: ...

class Holidays:
    year: int
    def __init__(self, year: int | None = None) -> None: ...
    def set_year(self, year: int) -> None: ...
    @property
    def christmas(self) -> tuple[int, int, int]: ...
    @property
    def christmas_eve(self) -> tuple[int, int, int]: ...
    @property
    def thanksgiving(self) -> tuple[int, int, int]: ...
    @property
    def new_years(self) -> tuple[int, int, int]: ...
    @property
    def new_years_eve(self) -> tuple[int, int, int]: ...
    @property
    def independence_day(self) -> tuple[int, int, int]: ...
    @property
    def flag_day(self) -> tuple[int, int, int]: ...
    @property
    def election_day(self) -> tuple[int, int, int]: ...
    @property
    def presidents_day(self) -> tuple[int, int, int]: ...
    @property
    def washingtons_birthday(self) -> tuple[int, int, int]: ...
    @property
    def lincolns_birthday(self) -> tuple[int, int, int]: ...
    @property
    def memorial_day(self) -> tuple[int, int, int]: ...
    @property
    def juneteenth(self) -> tuple[int, int, int]: ...
    @property
    def labor_day(self) -> tuple[int, int, int]: ...
    @property
    def indigenous_peoples_day(self) -> tuple[int, int, int]: ...
    @property
    def columbus_day(self) -> tuple[int, int, int]: ...
    @property
    def veterans_day(self) -> tuple[int, int, int]: ...
    @property
    def valentines_day(self) -> tuple[int, int, int]: ...
    @property
    def halloween(self) -> tuple[int, int, int]: ...
    @property
    def mothers_day(self) -> tuple[int, int, int]: ...
    @property
    def fathers_day(self) -> tuple[int, int, int]: ...
    @property
    def pulaski_day(self) -> tuple[int, int, int]: ...
    @property
    def easter(self) -> tuple[int, int, int]: ...
    @property
    def martin_luther_king_day(self) -> tuple[int, int, int]: ...
    @property
    def hanukkah(self) -> tuple[int, int, int]: ...
    @property
    def purim(self) -> tuple[int, int, int]: ...
    @property
    def rosh_hashanah(self) -> tuple[int, int, int]: ...
    @property
    def yom_kippur(self) -> tuple[int, int, int]: ...
    @property
    def passover(self) -> tuple[int, int, int]: ...
    @property
    def shavuot(self) -> tuple[int, int, int]: ...
    @property
    def sukkot(self) -> tuple[int, int, int]: ...
    @property
    def tu_beshvat(self) -> tuple[int, int, int]: ...
    @property
    def shemini_azeret(self) -> tuple[int, int, int]: ...
    @property
    def lag_baomer(self) -> tuple[int, int, int]: ...
    @property
    def tisha_bav(self) -> tuple[int, int, int]: ...
    @property
    def dia_constitucion(self) -> tuple[int, int, int]: ...
    @property
    def natalicio_benito_juarez(self) -> tuple[int, int, int]: ...
    @property
    def dia_independencia(self) -> tuple[int, int, int]: ...
    @property
    def dia_revolucion(self) -> tuple[int, int, int]: ...
    @property
    def ramadan(self) -> tuple[int, int, int]: ...
    @property
    def ashura(self) -> tuple[int, int, int]: ...
    @property
    def eid_alfitr(self) -> tuple[int, int, int]: ...
    @property
    def eid_aladha(self) -> tuple[int, int, int]: ...



from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["capacity_scaling"]

@_dispatchable
def capacity_scaling(
    G: Graph[_Node], demand: str = "demand", capacity: str = "capacity", weight: str = "weight", heap: type = ...
): ...



from collections.abc import Generator
from io import StringIO
from logging import Logger
from typing import IO, Any, TypedDict, type_check_only

log: Logger

@type_check_only
class _ContainerLimits(TypedDict, total=False):
    memory: int
    memswap: int
    cpushares: int
    cpusetcpus: str

@type_check_only
class _Filers(TypedDict, total=False):
    dangling: bool
    until: str

class BuildApiMixin:
    def build(
        self,
        path: str | None = None,
        tag: str | None = None,
        quiet: bool = False,
        fileobj: StringIO | IO[bytes] | None = None,
        nocache: bool = False,
        rm: bool = False,
        timeout: int | None = None,
        custom_context: bool = False,
        encoding: str | None = None,
        pull: bool = False,
        forcerm: bool = False,
        dockerfile: str | None = None,
        container_limits: _ContainerLimits | None = None,
        decode: bool = False,
        buildargs: dict[str, Any] | None = None,
        gzip: bool = False,
        shmsize: int | None = None,
        labels: dict[str, Any] | None = None,
        # need to use list, because the type must be json serializable
        cache_from: list[str] | None = None,
        target: str | None = None,
        network_mode: str | None = None,
        squash: bool | None = None,
        extra_hosts: list[str] | dict[str, str] | None = None,
        platform: str | None = None,
        isolation: str | None = None,
        use_config_proxy: bool = True,
    ) -> Generator[Any, None, None]: ...
    def prune_builds(
        self, filters: _Filers | None = None, keep_storage: int | None = None, all: bool | None = None
    ) -> dict[str, Any]: ...

def process_dockerfile(dockerfile: str | None, path: str) -> tuple[str | None, str | None]: ...



from authlib.oauth2.rfc6749 import AuthorizationCodeMixin as _AuthorizationCodeMixin

class AuthorizationCodeMixin(_AuthorizationCodeMixin):
    def get_nonce(self) -> str | None: ...
    def get_auth_time(self) -> int | None: ...
    def get_acr(self) -> str: ...
    def get_amr(self) -> list[str]: ...



import inspect
import os
from _typeshed import Incomplete, SupportsKeysAndGetItem
from collections.abc import Callable, Generator
from contextlib import contextmanager
from typing import IO, Any, Literal, NoReturn, TypedDict, TypeVar
from typing_extensions import Never, Self

import matplotlib as mpl
from matplotlib.artist import Artist
from matplotlib.axes import Axes
from matplotlib.figure import Figure, SubFigure
from matplotlib.transforms import Bbox
from matplotlib.typing import ColorType
from seaborn._core.data import PlotData
from seaborn._core.moves import Move
from seaborn._core.scales import Scale
from seaborn._core.typing import DataSource, Default, OrderSpec, VariableSpec, VariableSpecList
from seaborn._marks.base import Mark
from seaborn._stats.base import Stat

_ClsT = TypeVar("_ClsT", bound=type[Any])

default: Default

class Layer(TypedDict, total=False):
    mark: Mark
    stat: Stat | None
    move: Move | list[Move] | None
    data: PlotData
    source: DataSource
    vars: dict[str, VariableSpec]
    orient: str
    legend: bool
    label: str | None

class FacetSpec(TypedDict, total=False):
    variables: dict[str, VariableSpec]
    structure: dict[str, list[str]]
    wrap: int | None

class PairSpec(TypedDict, total=False):
    variables: dict[str, VariableSpec]
    structure: dict[str, list[str]]
    cross: bool
    wrap: int | None

@contextmanager
def theme_context(params: dict[str, Any]) -> Generator[None]: ...
def build_plot_signature(cls: _ClsT) -> _ClsT: ...  # -> _ClsT & "__signature__ protocol"

class ThemeConfig(mpl.RcParams):
    THEME_GROUPS: list[str]
    def __init__(self) -> None: ...
    def reset(self) -> None: ...
    def update(self, other: SupportsKeysAndGetItem[Incomplete, Incomplete] | None = None, /, **kwds) -> None: ...  # type: ignore[override]

class DisplayConfig(TypedDict):
    format: Literal["png", "svg"]
    scaling: float
    hidpi: bool

class PlotConfig:
    def __init__(self) -> None: ...
    @property
    def theme(self) -> dict[str, Any]: ...
    @property
    def display(self) -> DisplayConfig: ...

@build_plot_signature
class Plot:
    __signature__: inspect.Signature
    config: PlotConfig
    def __init__(self, *args: DataSource | VariableSpec, data: DataSource = None, **variables: VariableSpec) -> None: ...
    def __add__(self, other: Never) -> NoReturn: ...
    def on(self, target: Axes | SubFigure | Figure) -> Plot: ...
    def add(
        self,
        mark: Mark,
        *transforms: Stat | Move,
        orient: str | None = None,
        legend: bool = True,
        label: str | None = None,
        data: DataSource = None,
        **variables: VariableSpec,
    ) -> Plot: ...
    def pair(
        self, x: VariableSpecList = None, y: VariableSpecList = None, wrap: int | None = None, cross: bool = True
    ) -> Plot: ...
    def facet(
        self,
        col: VariableSpec = None,
        row: VariableSpec = None,
        order: OrderSpec | dict[str, OrderSpec] = None,
        wrap: int | None = None,
    ) -> Plot: ...
    def scale(self, **scales: Scale) -> Plot: ...
    def share(self, **shares: bool | str) -> Plot: ...
    def limit(self, **limits: tuple[Any, Any]) -> Plot: ...
    def label(self, *, title: str | None = None, legend: str | None = None, **variables: str | Callable[[str], str]) -> Plot: ...
    def layout(
        self,
        *,
        size: tuple[float, float] | Default = ...,
        engine: str | None | Default = ...,
        extent: tuple[float, float, float, float] | Default = ...,
    ) -> Plot: ...
    def theme(self, config: dict[str, Any], /) -> Plot: ...
    # Same signature as Plotter.save
    def save(
        self,
        loc: str | os.PathLike[Any] | IO[Any],
        *,
        transparent: bool | None = None,
        dpi: float | None = 96,
        facecolor: ColorType | Literal["auto"] | None = ...,
        edgecolor: ColorType | Literal["auto"] | None = ...,
        orientation: str = "protrait",
        format: str | None = None,
        bbox_inches: Literal["tight"] | Bbox | None = None,
        pad_inches: float | None = None,
        bbox_extra_artists: list[Artist] | None = None,
        backend: str | None = None,
        **kwargs: Any,
    ) -> Self: ...
    # Same signature as Plotter.show
    def show(self, *, block: bool | None = None) -> None: ...
    def plot(self, pyplot: bool = False) -> Plotter: ...

class Plotter:
    def __init__(self, pyplot: bool, theme: dict[str, Any]) -> None: ...
    def save(
        self,
        loc: str | os.PathLike[Any] | IO[Any],
        *,
        # From matplotlib.figure.Figure.savefig
        transparent: bool | None = None,
        # keyword-only arguments below are the same as matplotlib.backend_bases.FigureCanvasBase.print_figure
        # but with different defaults
        dpi: float | None = 96,
        facecolor: ColorType | Literal["auto"] | None = ...,
        edgecolor: ColorType | Literal["auto"] | None = ...,
        orientation: str = "protrait",
        format: str | None = None,
        bbox_inches: Literal["tight"] | Bbox | None = None,
        pad_inches: float | None = None,
        bbox_extra_artists: list[Artist] | None = None,
        backend: str | None = None,
        # Further **kwargs can truly be anything from an overridden Canvas method that is still passed down
        **kwargs: Any,
    ) -> Self: ...
    # Same as matplotlib.backend_bases._Bases. No other backend override show
    def show(self, *, block: bool | None = None) -> None: ...



from collections.abc import Callable
from typing import Any
from typing_extensions import TypeAlias

import numpy as np
from tensorflow import Tensor
from tensorflow._aliases import FloatArray, FloatDataSequence, FloatTensorCompatible, Integer

# The implementation uses isinstance so it must be dict and not any Mapping.
_Activation: TypeAlias = str | None | Callable[[Tensor], Tensor] | dict[str, Any]
# Ints are not allowed.
_ActivationInput: TypeAlias = Tensor | FloatDataSequence | FloatArray | np.number[Any] | float

def deserialize(config: dict[str, Any], custom_objects: dict[str, Callable[..., Any]] | None = None) -> Callable[..., Any]: ...
def elu(x: _ActivationInput, alpha: FloatTensorCompatible | FloatDataSequence = 1.0) -> Tensor: ...
def exponential(x: _ActivationInput) -> Tensor: ...
def gelu(x: _ActivationInput, approximate: bool = False) -> Tensor: ...
def get(identifier: _Activation) -> Callable[[Tensor], Tensor]: ...
def hard_sigmoid(x: _ActivationInput) -> Tensor: ...
def linear(x: _ActivationInput) -> Tensor: ...
def mish(x: _ActivationInput) -> Tensor: ...
def relu(
    x: _ActivationInput,
    negative_slope: FloatTensorCompatible = 0.0,
    max_value: FloatTensorCompatible | FloatDataSequence | None = None,
    threshold: FloatTensorCompatible | FloatDataSequence = 0.0,
) -> Tensor: ...
def selu(x: _ActivationInput) -> Tensor: ...
def serialize(activation: Callable[..., Any]) -> str | dict[str, Any]: ...
def sigmoid(x: _ActivationInput) -> Tensor: ...
def softmax(x: Tensor, axis: Integer = -1) -> Tensor: ...
def softplus(x: _ActivationInput) -> Tensor: ...
def softsign(x: _ActivationInput) -> Tensor: ...
def swish(x: _ActivationInput) -> Tensor: ...
def tanh(x: _ActivationInput) -> Tensor: ...



from email.message import Message

__all__ = ["encode_7or8bit", "encode_base64", "encode_noop", "encode_quopri"]

def encode_base64(msg: Message) -> None: ...
def encode_quopri(msg: Message) -> None: ...
def encode_7or8bit(msg: Message) -> None: ...
def encode_noop(msg: Message) -> None: ...



from networkx.classes.digraph import DiGraph
from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["is_arborescence", "is_branching", "is_forest", "is_tree"]

@_dispatchable
def is_arborescence(G: Graph[_Node]) -> bool: ...
@_dispatchable
def is_branching(G: DiGraph[_Node]) -> bool: ...
@_dispatchable
def is_forest(G: Graph[_Node]) -> bool: ...
@_dispatchable
def is_tree(G: Graph[_Node]) -> bool: ...



ERROR_FIRST: int
MIN_ERROR: int
UNKNOWN_ERROR: int
SOCKET_CREATE_ERROR: int
CONNECTION_ERROR: int
CONN_HOST_ERROR: int
IPSOCK_ERROR: int
UNKNOWN_HOST: int
SERVER_GONE_ERROR: int
VERSION_ERROR: int
OUT_OF_MEMORY: int
WRONG_HOST_INFO: int
LOCALHOST_CONNECTION: int
TCP_CONNECTION: int
SERVER_HANDSHAKE_ERR: int
SERVER_LOST: int
COMMANDS_OUT_OF_SYNC: int
NAMEDPIPE_CONNECTION: int
NAMEDPIPEWAIT_ERROR: int
NAMEDPIPEOPEN_ERROR: int
NAMEDPIPESETSTATE_ERROR: int
CANT_READ_CHARSET: int
NET_PACKET_TOO_LARGE: int
EMBEDDED_CONNECTION: int
PROBE_SLAVE_STATUS: int
PROBE_SLAVE_HOSTS: int
PROBE_SLAVE_CONNECT: int
PROBE_MASTER_CONNECT: int
SSL_CONNECTION_ERROR: int
MALFORMED_PACKET: int
WRONG_LICENSE: int
NULL_POINTER: int
NO_PREPARE_STMT: int
PARAMS_NOT_BOUND: int
DATA_TRUNCATED: int
NO_PARAMETERS_EXISTS: int
INVALID_PARAMETER_NO: int
INVALID_BUFFER_USE: int
UNSUPPORTED_PARAM_TYPE: int
SHARED_MEMORY_CONNECTION: int
SHARED_MEMORY_CONNECT_REQUEST_ERROR: int
SHARED_MEMORY_CONNECT_ANSWER_ERROR: int
SHARED_MEMORY_CONNECT_FILE_MAP_ERROR: int
SHARED_MEMORY_CONNECT_MAP_ERROR: int
SHARED_MEMORY_FILE_MAP_ERROR: int
SHARED_MEMORY_MAP_ERROR: int
SHARED_MEMORY_EVENT_ERROR: int
SHARED_MEMORY_CONNECT_ABANDONED_ERROR: int
SHARED_MEMORY_CONNECT_SET_ERROR: int
CONN_UNKNOW_PROTOCOL: int
INVALID_CONN_HANDLE: int
UNUSED_1: int
FETCH_CANCELED: int
NO_DATA: int
NO_STMT_METADATA: int
NO_RESULT_SET: int
NOT_IMPLEMENTED: int
SERVER_LOST_EXTENDED: int
STMT_CLOSED: int
NEW_STMT_METADATA: int
ALREADY_CONNECTED: int
AUTH_PLUGIN_CANNOT_LOAD: int
DUPLICATE_CONNECTION_ATTR: int
AUTH_PLUGIN_ERR: int
INSECURE_API_ERR: int
FILE_NAME_TOO_LONG: int
SSL_FIPS_MODE_ERR: int
MAX_ERROR: int
ERROR_LAST: int



from _collections_abc import dict_keys
from _typeshed import Incomplete
from collections.abc import Iterable
from re import Pattern
from typing import AnyStr, ClassVar, Final, overload
from typing_extensions import Self

from .caselessdict import CaselessDict
from .parser_tools import ICAL_TYPE

__all__ = [
    "Contentline",
    "Contentlines",
    "FOLD",
    "NAME",
    "NEWLINE",
    "Parameters",
    "QUNSAFE_CHAR",
    "QUOTABLE",
    "UNSAFE_CHAR",
    "dquote",
    "escape_char",
    "escape_string",
    "foldline",
    "param_value",
    "q_join",
    "q_split",
    "rfc_6868_escape",
    "rfc_6868_unescape",
    "uFOLD",
    "unescape_char",
    "unescape_list_or_string",
    "unescape_string",
    "validate_param_value",
    "validate_token",
]

def escape_char(text: str) -> str: ...
def unescape_char(text: AnyStr) -> AnyStr: ...
def foldline(line: str, limit: int = 75, fold_sep: str = "\r\n ") -> str: ...
def param_value(value: str | list[str] | tuple[str, ...] | Incomplete, always_quote: bool = False) -> str: ...

NAME: Final[Pattern[str]]
UNSAFE_CHAR: Final[Pattern[str]]
QUNSAFE_CHAR: Final[Pattern[str]]
FOLD: Final[Pattern[bytes]]
uFOLD: Final[Pattern[str]]
NEWLINE: Final[Pattern[str]]

def validate_token(name: str) -> None: ...
def validate_param_value(value: str, quoted: bool = True) -> None: ...

QUOTABLE: Final[Pattern[str]]

def dquote(val: str, always_quote: bool = False) -> str: ...
def q_split(st: str, sep: str = ",", maxsplit: int = -1) -> list[str]: ...
def q_join(lst: Iterable[str], sep: str = ",", always_quote: bool = False) -> str: ...

class Parameters(CaselessDict[str]):
    always_quoted: ClassVar[tuple[str, ...]]
    quote_also: ClassVar[dict[str, str]]
    def params(self) -> dict_keys[str, str]: ...
    def to_ical(self, sorted: bool = True) -> bytes: ...
    @classmethod
    def from_ical(cls, st: str, strict: bool = False) -> Self: ...

def escape_string(val: str) -> str: ...
def unescape_string(val: str) -> str: ...

RFC_6868_UNESCAPE_REGEX: Final[Pattern[str]]

def rfc_6868_unescape(param_value: str) -> str: ...

RFC_6868_ESCAPE_REGEX: Final[Pattern[str]]

def rfc_6868_escape(param_value: str) -> str: ...
@overload
def unescape_list_or_string(val: list[str]) -> list[str]: ...
@overload
def unescape_list_or_string(val: str) -> str: ...

class Contentline(str):
    strict: bool
    def __new__(cls, value: str | bytes, strict: bool = False, encoding: str = "utf-8") -> Self: ...
    @classmethod
    def from_parts(cls, name: ICAL_TYPE, params: Parameters, values, sorted: bool = True) -> Self: ...
    def parts(self) -> tuple[str, Parameters, str]: ...
    @classmethod
    def from_ical(cls, ical: str | bytes, strict: bool = False) -> Self: ...
    def to_ical(self) -> bytes: ...

class Contentlines(list[Contentline]):
    def to_ical(self) -> bytes: ...
    @classmethod
    def from_ical(cls, st: str | bytes) -> Self: ...



from _typeshed import Incomplete
from typing import Final

from braintree.error_result import ErrorResult
from braintree.successful_result import SuccessfulResult

class BankAccountInstantVerificationGateway:
    gateway: Incomplete
    config: Incomplete
    graphql_client: Incomplete
    CREATE_JWT_MUTATION: Final[str]
    def __init__(self, gateway) -> None: ...
    def create_jwt(self, request) -> SuccessfulResult | ErrorResult: ...



from typing import Final

from pymeeus.Angle import Angle
from pymeeus.Epoch import Epoch

VSOP87_L: Final[list[list[list[float]]]]
VSOP87_B: Final[list[list[list[float]]]]
VSOP87_R: Final[list[list[list[float]]]]
ORBITAL_ELEM: Final[list[list[float]]]
ORBITAL_ELEM_J2000: Final[list[list[float]]]

class Jupiter:
    @staticmethod
    def geometric_heliocentric_position(epoch: Epoch, tofk5: bool | None = True) -> tuple[Angle, Angle, float]: ...
    @staticmethod
    def apparent_heliocentric_position(epoch: Epoch) -> tuple[Angle, Angle, float]: ...
    @staticmethod
    def orbital_elements_mean_equinox(epoch: Epoch) -> tuple[Angle, float, float, Angle, Angle, Angle]: ...
    @staticmethod
    def orbital_elements_j2000(epoch: Epoch) -> tuple[Angle, float, float, Angle, Angle, Angle]: ...
    @staticmethod
    def geocentric_position(epoch: Epoch) -> tuple[Angle, Angle, Angle]: ...
    @staticmethod
    def conjunction(epoch: Epoch) -> Epoch: ...
    @staticmethod
    def opposition(epoch: Epoch) -> Epoch: ...
    @staticmethod
    def station_longitude_1(epoch: Epoch) -> Epoch: ...
    @staticmethod
    def station_longitude_2(epoch: Epoch) -> Epoch: ...
    @staticmethod
    def perihelion_aphelion(epoch: Epoch, perihelion: bool | None = True) -> Epoch: ...
    @staticmethod
    def passage_nodes(epoch: Epoch, ascending: bool = True) -> tuple[Epoch, float]: ...
    @staticmethod
    def magnitude(sun_dist: float, earth_dist: float) -> float: ...

def main() -> None: ...



from _typeshed import Incomplete

IConnectionPointContainer_methods: Incomplete
IConnectionPoint_methods: Incomplete

class ConnectableServer:
    cookieNo: int
    connections: Incomplete
    def EnumConnections(self) -> None: ...
    def GetConnectionInterface(self) -> None: ...
    def GetConnectionPointContainer(self): ...
    def Advise(self, pUnk): ...
    def Unadvise(self, cookie) -> None: ...
    def EnumConnectionPoints(self) -> None: ...
    def FindConnectionPoint(self, iid): ...



from typing import Final

from hvac.api.vault_api_base import VaultApiBase

DEFAULT_MOUNT_POINT: Final = "ldap"

class Ldap(VaultApiBase):
    def configure(
        self,
        binddn: str | None = None,
        bindpass: str | None = None,
        url: str | None = None,
        password_policy: str | None = None,
        schema: str | None = None,
        userdn: str | None = None,
        userattr: str | None = None,
        upndomain: str | None = None,
        connection_timeout: int | str | None = None,
        request_timeout: int | str | None = None,
        starttls: bool | None = None,
        insecure_tls: bool | None = None,
        certificate: str | None = None,
        client_tls_cert: str | None = None,
        client_tls_key: str | None = None,
        mount_point: str = "ldap",
    ): ...
    def read_config(self, mount_point: str = "ldap"): ...
    def rotate_root(self, mount_point: str = "ldap"): ...
    def create_or_update_static_role(
        self,
        name: str,
        username: str | None = None,
        dn: str | None = None,
        rotation_period: str | None = None,
        mount_point: str = "ldap",
    ): ...
    def read_static_role(self, name: str, mount_point: str = "ldap"): ...
    def list_static_roles(self, mount_point: str = "ldap"): ...
    def delete_static_role(self, name: str, mount_point: str = "ldap"): ...
    def generate_static_credentials(self, name: str, mount_point: str = "ldap"): ...
    def rotate_static_credentials(self, name: str, mount_point: str = "ldap"): ...



from collections.abc import Callable
from typing import Generic, TypeVar

_T = TypeVar("_T")

def construct_format_field_str(fname: str | None, fspec: str | None, conv: str | None) -> str: ...
def infer_positional_format_args(fstr: str) -> str: ...
def get_format_args(fstr: str) -> tuple[list[tuple[int, type]], list[tuple[str, type]]]: ...
def tokenize_format_str(fstr: str, resolve_pos: bool = True) -> list[str | BaseFormatField]: ...

class BaseFormatField:
    def __init__(self, fname: str, fspec: str = "", conv: str | None = None) -> None: ...
    base_name: str
    fname: str
    subpath: str
    is_positional: bool
    def set_fname(self, fname: str) -> None: ...
    subfields: list[str]
    fspec: str
    type_char: str
    type_func: str
    def set_fspec(self, fspec) -> None: ...
    conv: str | None
    conv_func: str | None
    def set_conv(self, conv: str | None) -> None: ...
    @property
    def fstr(self) -> str: ...

class DeferredValue(Generic[_T]):
    func: Callable[[], _T]
    cache_value: bool
    def __init__(self, func: Callable[[], _T], cache_value: bool = True) -> None: ...
    def get_value(self) -> _T: ...
    def __int__(self) -> int: ...
    def __float__(self) -> float: ...
    def __unicode__(self) -> str: ...
    def __format__(self, fmt: str) -> str: ...

__all__ = [
    "DeferredValue",
    "get_format_args",
    "tokenize_format_str",
    "construct_format_field_str",
    "infer_positional_format_args",
    "BaseFormatField",
]



from _typeshed import ReadableBuffer
from collections.abc import Iterable
from typing import Any, SupportsBytes, SupportsIndex, overload

@overload
def to_bytes(x: None, charset: str = "utf-8", errors: str = "strict") -> None: ...
@overload
def to_bytes(
    x: str | bytes | float | Iterable[SupportsIndex] | SupportsIndex | SupportsBytes | ReadableBuffer,
    charset: str = "utf-8",
    errors: str = "strict",
) -> bytes: ...
@overload
def to_unicode(x: None, charset: str = "utf-8", errors: str = "strict") -> None: ...
@overload
def to_unicode(x: object, charset: str = "utf-8", errors: str = "strict") -> str: ...
def to_native(x: str | bytes, encoding: str = "ascii") -> str: ...
def json_loads(s: str | bytes | bytearray) -> Any: ...  # returns json.loads()
def json_dumps(data: Any, ensure_ascii: bool = False) -> str: ...  # data pass to json.dumps()
def urlsafe_b64decode(s: bytes) -> bytes: ...
def urlsafe_b64encode(s: ReadableBuffer) -> bytes: ...
def base64_to_int(s: str | bytes | float | Iterable[SupportsIndex] | SupportsIndex | SupportsBytes | ReadableBuffer) -> int: ...
def int_to_base64(num: int) -> str: ...
def json_b64encode(
    text: str | bytes | float | Iterable[SupportsIndex] | SupportsIndex | SupportsBytes | ReadableBuffer,
) -> bytes: ...



from _typeshed import Incomplete
from typing import Final

from reportlab.pdfbase import pdfmetrics

__version__: Final[str]
DISABLE_CMAP: bool

def findCMapFile(name): ...
def structToPDF(structure): ...

class CIDEncoding(pdfmetrics.Encoding):
    name: Incomplete
    source: Incomplete
    def __init__(self, name, useCache: int = 1) -> None: ...
    def parseCMAPFile(self, name) -> None: ...
    def translate(self, text): ...
    def fastSave(self, directory) -> None: ...
    def fastLoad(self, directory) -> None: ...
    def getData(self): ...

class CIDTypeFace(pdfmetrics.TypeFace):
    def __init__(self, name) -> None: ...
    def getCharWidth(self, characterId): ...

class CIDFont(pdfmetrics.Font):
    faceName: Incomplete
    face: Incomplete
    encodingName: Incomplete
    encoding: Incomplete
    fontName: Incomplete
    name: Incomplete
    isVertical: Incomplete
    substitutionFonts: Incomplete
    def __init__(self, face, encoding) -> None: ...
    def formatForPdf(self, text): ...
    def stringWidth(self, text, size, encoding=None): ...
    def addObjects(self, doc) -> None: ...

class UnicodeCIDFont(CIDFont):
    language: Incomplete
    name: Incomplete
    vertical: Incomplete
    isHalfWidth: Incomplete
    unicodeWidths: Incomplete
    def __init__(self, face, isVertical: bool = False, isHalfWidth: bool = False) -> None: ...
    def formatForPdf(self, text): ...
    def stringWidth(self, text, size, encoding=None): ...

def precalculate(cmapdir) -> None: ...
def test() -> None: ...



from _typeshed import Incomplete
from typing import Any

from google.cloud.ndb import tasklets as tasklets

class ContextCache:
    def get_and_validate(self, key): ...

class _GlobalCacheBatch:
    def full(self): ...
    def idle_callback(self) -> None: ...
    def done_callback(self, cache_call) -> None: ...
    def make_call(self) -> None: ...
    def future_info(self, key) -> None: ...

global_get: Any

class _GlobalCacheGetBatch(_GlobalCacheBatch):
    todo: Any
    keys: Any
    def __init__(self, ignore_options) -> None: ...
    def add(self, key): ...
    def done_callback(self, cache_call) -> None: ...
    def make_call(self): ...
    def future_info(self, key): ...

def global_set(key, value, expires: Incomplete | None = ..., read: bool = ...): ...

class _GlobalCacheSetBatch(_GlobalCacheBatch):
    expires: Any
    todo: object
    futures: object
    def __init__(self, options) -> None: ...
    def done_callback(self, cache_call) -> None: ...
    def add(self, key, value): ...
    def make_call(self): ...
    def future_info(self, key, value): ...  # type:ignore[override]

class _GlobalCacheSetIfNotExistsBatch(_GlobalCacheSetBatch):
    def add(self, key, value): ...
    def make_call(self): ...
    def future_info(self, key, value): ...  # type:ignore[override]

global_delete: Any

class _GlobalCacheDeleteBatch(_GlobalCacheBatch):
    keys: Any
    futures: Any
    def __init__(self, ignore_options) -> None: ...
    def add(self, key): ...
    def make_call(self): ...
    def future_info(self, key): ...

global_watch: Any

class _GlobalCacheWatchBatch(_GlobalCacheDeleteBatch):
    def make_call(self): ...
    def future_info(self, key, value): ...  # type:ignore[override]

def global_unwatch(key): ...

class _GlobalCacheUnwatchBatch(_GlobalCacheDeleteBatch):
    def make_call(self): ...
    def future_info(self, key): ...

global_compare_and_swap: Any

class _GlobalCacheCompareAndSwapBatch(_GlobalCacheSetBatch):
    def make_call(self): ...
    def future_info(self, key, value): ...  # type:ignore[override]

def is_locked_value(value): ...
def global_cache_key(key): ...



import grpc

from . import _async as aio

# Returns a google.rpc.status.Status message corresponding to a given grpc.Call.
def from_call(call: grpc.Call): ...

# Convert a google.rpc.status.Status message to grpc.Status.
def to_status(status) -> grpc.Status: ...

__all__ = ["from_call", "to_status", "aio"]



from _typeshed import Incomplete
from collections.abc import Callable, Coroutine, Iterable, Mapping, Sequence
from contextlib import AbstractContextManager
from types import TracebackType
from typing import Any, ClassVar, Generic, Literal, TypeVar, overload, type_check_only
from typing_extensions import ParamSpec, Self

_F = TypeVar("_F", bound=Callable[..., Any])
_AF = TypeVar("_AF", bound=Callable[..., Coroutine[Any, Any, Any]])
_T = TypeVar("_T")
_TT = TypeVar("_TT", bound=type[Any])
_R = TypeVar("_R")
_P = ParamSpec("_P")

__all__ = (
    "Mock",
    "MagicMock",
    "patch",
    "sentinel",
    "DEFAULT",
    "ANY",
    "call",
    "create_autospec",
    "AsyncMock",
    "ThreadingMock",
    "FILTER_DIR",
    "NonCallableMock",
    "NonCallableMagicMock",
    "mock_open",
    "PropertyMock",
    "seal",
)

class InvalidSpecError(Exception): ...

FILTER_DIR: bool

class _SentinelObject:
    def __init__(self, name: str) -> None: ...
    name: str

class _Sentinel:
    def __getattr__(self, name: str) -> _SentinelObject: ...

sentinel: _Sentinel
DEFAULT: _SentinelObject

class _Call(tuple[Any, ...]):
    def __new__(
        cls, value: Any = (), name: Incomplete | None = "", parent=None, two: bool = False, from_kall: bool = True
    ) -> Self: ...
    name: Any
    parent: Any
    from_kall: Any
    def __init__(self, value: Any = (), name=None, parent=None, two: bool = False, from_kall: bool = True) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __ne__(self, other: object, /) -> bool: ...
    def __call__(self, *args: Any, **kwargs: Any) -> _Call: ...
    def __getattr__(self, attr: str) -> Any: ...
    @property
    def args(self) -> tuple[Any, ...]: ...
    @property
    def kwargs(self) -> dict[str, Any]: ...
    def call_list(self) -> _CallList: ...

call: _Call

class _CallList(list[_Call]):
    def __contains__(self, value: Any) -> bool: ...

class Base:
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

# We subclass with "Any" because mocks are explicitly designed to stand in for other types,
# something that can't be expressed with our static type system.
class NonCallableMock(Base, Any):
    def __new__(
        cls,
        spec: list[str] | object | type[object] | None = None,
        wraps: Any | None = None,
        name: str | None = None,
        spec_set: list[str] | object | type[object] | None = None,
        parent: NonCallableMock | None = None,
        _spec_state=None,
        _new_name: str = "",
        _new_parent: NonCallableMock | None = None,
        _spec_as_instance: bool = False,
        _eat_self: bool | None = None,
        unsafe: bool = False,
        **kwargs: Any,
    ) -> Self: ...
    def __init__(
        self,
        spec: list[str] | object | type[object] | None = None,
        wraps: Any | None = None,
        name: str | None = None,
        spec_set: list[str] | object | type[object] | None = None,
        parent: NonCallableMock | None = None,
        _spec_state=None,
        _new_name: str = "",
        _new_parent: NonCallableMock | None = None,
        _spec_as_instance: bool = False,
        _eat_self: bool | None = None,
        unsafe: bool = False,
        **kwargs: Any,
    ) -> None: ...
    def __getattr__(self, name: str) -> Any: ...
    def _calls_repr(self) -> str: ...
    def assert_called_with(_mock_self, *args: Any, **kwargs: Any) -> None: ...
    def assert_not_called(_mock_self) -> None: ...
    def assert_called_once_with(_mock_self, *args: Any, **kwargs: Any) -> None: ...
    def _format_mock_failure_message(self, args: Any, kwargs: Any, action: str = "call") -> str: ...
    def assert_called(_mock_self) -> None: ...
    def assert_called_once(_mock_self) -> None: ...
    def reset_mock(self, visited: Any = None, *, return_value: bool = False, side_effect: bool = False) -> None: ...
    def _extract_mock_name(self) -> str: ...
    def assert_any_call(self, *args: Any, **kwargs: Any) -> None: ...
    def assert_has_calls(self, calls: Sequence[_Call], any_order: bool = False) -> None: ...
    def mock_add_spec(self, spec: Any, spec_set: bool = False) -> None: ...
    def _mock_add_spec(self, spec: Any, spec_set: bool, _spec_as_instance: bool = False, _eat_self: bool = False) -> None: ...
    def attach_mock(self, mock: NonCallableMock, attribute: str) -> None: ...
    def configure_mock(self, **kwargs: Any) -> None: ...
    return_value: Any
    side_effect: Any
    called: bool
    call_count: int
    call_args: Any
    call_args_list: _CallList
    mock_calls: _CallList
    def _format_mock_call_signature(self, args: Any, kwargs: Any) -> str: ...
    def _call_matcher(self, _call: tuple[_Call, ...]) -> _Call: ...
    def _get_child_mock(self, **kw: Any) -> NonCallableMock: ...

class CallableMixin(Base):
    side_effect: Any
    def __init__(
        self,
        spec=None,
        side_effect=None,
        return_value: Any = ...,
        wraps=None,
        name=None,
        spec_set=None,
        parent=None,
        _spec_state=None,
        _new_name: Any = "",
        _new_parent=None,
        **kwargs: Any,
    ) -> None: ...
    def __call__(_mock_self, *args: Any, **kwargs: Any) -> Any: ...

class Mock(CallableMixin, NonCallableMock): ...

class _patch(Generic[_T]):
    attribute_name: Any
    getter: Callable[[], Any]
    attribute: str
    new: _T
    new_callable: Any
    spec: Any
    create: bool
    has_local: Any
    spec_set: Any
    autospec: Any
    kwargs: Mapping[str, Any]
    additional_patchers: Any
    def __init__(
        self: _patch[_T],  # pyright: ignore[reportInvalidTypeVarUse]  #11780
        getter: Callable[[], Any],
        attribute: str,
        new: _T,
        spec: Incomplete | None,
        create: bool,
        spec_set: Incomplete | None,
        autospec: Incomplete | None,
        new_callable: Incomplete | None,
        kwargs: Mapping[str, Any],
        *,
        unsafe: bool = False,
    ) -> None: ...
    def copy(self) -> _patch[_T]: ...
    def __call__(self, func: Callable[_P, _R]) -> Callable[_P, _R]: ...
    def decorate_class(self, klass: _TT) -> _TT: ...
    def decorate_callable(self, func: _F) -> _F: ...
    def decorate_async_callable(self, func: _AF) -> _AF: ...
    def decoration_helper(
        self, patched: Any, args: tuple[Any, ...], keywargs: dict[str, Any]
    ) -> AbstractContextManager[tuple[tuple[Any, ...], dict[str, Any]]]: ...
    def get_original(self) -> tuple[Any, bool]: ...
    target: Any
    temp_original: Any
    is_local: bool
    def __enter__(self) -> _T: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_value: BaseException | None, traceback: TracebackType | None, /
    ) -> None: ...
    def start(self) -> _T: ...
    def stop(self) -> None: ...

class _patch_dict:
    in_dict: Any
    values: Any
    clear: Any
    def __init__(self, in_dict: Any, values: Any = (), clear: Any = False, **kwargs: Any) -> None: ...
    def __call__(self, f: Any) -> Any: ...
    def decorate_callable(self, f: _F) -> _F: ...
    def decorate_async_callable(self, f: _AF) -> _AF: ...
    def decorate_class(self, klass: Any) -> Any: ...
    def __enter__(self) -> Any: ...
    def __exit__(self, *args: object) -> Any: ...
    start: Any
    stop: Any

@type_check_only
class _patcher:
    TEST_PREFIX: str
    dict: type[_patch_dict]
    @overload
    def __call__(
        self,
        target: Any,
        *,
        spec: Incomplete | None = ...,
        create: bool = ...,
        spec_set: Incomplete | None = ...,
        autospec: Incomplete | None = ...,
        new_callable: Incomplete | None = ...,
        unsafe: bool = ...,
        **kwargs: Any,
    ) -> _patch[MagicMock | AsyncMock]: ...
    # This overload also covers the case, where new==DEFAULT. In this case, the return type is _patch[Any].
    # Ideally we'd be able to add an overload for it so that the return type is _patch[MagicMock],
    # but that's impossible with the current type system.
    @overload
    def __call__(
        self,
        target: Any,
        new: _T,
        spec: Incomplete | None = ...,
        create: bool = ...,
        spec_set: Incomplete | None = ...,
        autospec: Incomplete | None = ...,
        new_callable: Incomplete | None = ...,
        *,
        unsafe: bool = ...,
        **kwargs: Any,
    ) -> _patch[_T]: ...
    @overload
    def object(
        self,
        target: Any,
        attribute: str,
        *,
        spec: Incomplete | None = ...,
        create: bool = ...,
        spec_set: Incomplete | None = ...,
        autospec: Incomplete | None = ...,
        new_callable: Incomplete | None = ...,
        unsafe: bool = ...,
        **kwargs: Any,
    ) -> _patch[MagicMock | AsyncMock]: ...
    @overload
    def object(
        self,
        target: Any,
        attribute: str,
        new: _T,
        spec: Incomplete | None = ...,
        create: bool = ...,
        spec_set: Incomplete | None = ...,
        autospec: Incomplete | None = ...,
        new_callable: Incomplete | None = ...,
        *,
        unsafe: bool = ...,
        **kwargs: Any,
    ) -> _patch[_T]: ...
    def multiple(
        self,
        target: Any,
        spec: Incomplete | None = ...,
        create: bool = ...,
        spec_set: Incomplete | None = ...,
        autospec: Incomplete | None = ...,
        new_callable: Incomplete | None = ...,
        *,
        unsafe: bool = ...,
        **kwargs: _T,
    ) -> _patch[_T]: ...
    def stopall(self) -> None: ...

patch: _patcher

class MagicMixin:
    def __init__(self, *args: Any, **kw: Any) -> None: ...

class NonCallableMagicMock(MagicMixin, NonCallableMock):
    def mock_add_spec(self, spec: Any, spec_set: bool = False) -> None: ...

class MagicMock(MagicMixin, Mock):
    def mock_add_spec(self, spec: Any, spec_set: bool = False) -> None: ...

class AsyncMockMixin(Base):
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def assert_awaited(_mock_self) -> None: ...
    def assert_awaited_once(_mock_self) -> None: ...
    def assert_awaited_with(_mock_self, *args: Any, **kwargs: Any) -> None: ...
    def assert_awaited_once_with(_mock_self, *args: Any, **kwargs: Any) -> None: ...
    def assert_any_await(_mock_self, *args: Any, **kwargs: Any) -> None: ...
    def assert_has_awaits(_mock_self, calls: Iterable[_Call], any_order: bool = False) -> None: ...
    def assert_not_awaited(_mock_self) -> None: ...
    def reset_mock(self, *args: Any, **kwargs: Any) -> None: ...
    await_count: int
    await_args: _Call | None
    await_args_list: _CallList
    __name__: str
    __defaults__: tuple[Any, ...]
    __kwdefaults__: dict[str, Any]
    __annotations__: dict[str, Any] | None  # type: ignore[assignment]

class AsyncMagicMixin(MagicMixin): ...

class AsyncMock(AsyncMockMixin, AsyncMagicMixin, Mock):
    # Improving the `reset_mock` signature.
    # It is defined on `AsyncMockMixin` with `*args, **kwargs`, which is not ideal.
    # But, `NonCallableMock` super-class has the better version.
    def reset_mock(self, visited: Any = None, *, return_value: bool = False, side_effect: bool = False) -> None: ...

class MagicProxy(Base):
    name: str
    parent: Any
    def __init__(self, name: str, parent: Any) -> None: ...
    def create_mock(self) -> Any: ...
    def __get__(self, obj: Any, _type=None) -> Any: ...

class _ANY(Any):
    def __eq__(self, other: object) -> Literal[True]: ...
    def __ne__(self, other: object) -> Literal[False]: ...

ANY: _ANY

def create_autospec(
    spec: Any, spec_set: Any = False, instance: Any = False, _parent=None, _name=None, *, unsafe: bool = False, **kwargs: Any
) -> Any: ...

class _SpecState:
    spec: Any
    ids: Any
    spec_set: Any
    parent: Any
    instance: Any
    name: Any
    def __init__(self, spec: Any, spec_set: Any = False, parent=None, name=None, ids=None, instance: Any = False) -> None: ...

def mock_open(mock=None, read_data: Any = "") -> Any: ...

class PropertyMock(Mock):
    def __get__(self, obj: _T, obj_type: type[_T] | None = None) -> Self: ...
    def __set__(self, obj: Any, value: Any) -> None: ...

def seal(mock: Any) -> None: ...

class ThreadingMixin(Base):
    DEFAULT_TIMEOUT: ClassVar[float | None]

    def __init__(self, *args: Any, timeout: float | None = ..., **kwargs: Any) -> None: ...
    def reset_mock(self, *args: Any, **kwargs: Any) -> None: ...
    def wait_until_called(self, *, timeout: float | None = ...) -> None: ...
    def wait_until_any_call_with(self, *args: Any, **kwargs: Any) -> None: ...

class ThreadingMock(ThreadingMixin, MagicMixin, Mock):
    # Improving the `reset_mock` signature.
    # It is defined on `ThreadingMixin` with `*args, **kwargs`, which is not ideal.
    # But, `NonCallableMock` super-class has the better version.
    def reset_mock(self, visited: Any = None, *, return_value: bool = False, side_effect: bool = False) -> None: ...



from typing import Final

httplib_client_module: Final = "http.client"
PATCH_FLAG: Final = "__xray_patched"

def add_ignored(subclass=None, hostname=None, urls=None) -> None: ...
def reset_ignored() -> None: ...
def http_response_processor(wrapped, instance, args, kwargs, return_value, exception, subsegment, stack) -> None: ...
def http_send_request_processor(wrapped, instance, args, kwargs, return_value, exception, subsegment, stack) -> None: ...
def http_read_processor(wrapped, instance, args, kwargs, return_value, exception, subsegment, stack) -> None: ...
def patch() -> None: ...
def unpatch() -> None: ...



"""Jac compiler tools and parser generation utilities."""

import os
import sys

_vendor_dir = os.path.normpath(os.path.join(os.path.dirname(__file__), "..", "vendor"))
if _vendor_dir not in sys.path:
    sys.path.insert(0, _vendor_dir)

_cur_dir = os.path.dirname(__file__)
_pycore_dir = os.path.normpath(os.path.join(os.path.dirname(__file__), "..", "pycore"))


def generate_static_parser(force: bool = False) -> None:
    """Generate static parser for Jac."""
    from lark.tools import standalone

    lark_jac_parser_path = os.path.join(_pycore_dir, "lark_jac_parser.py")
    if force or not os.path.exists(lark_jac_parser_path):
        sys.argv, save = (
            [
                "lark",
                os.path.join(_pycore_dir, "jac.lark"),
                "-o",
                lark_jac_parser_path,
                "-c",
            ],
            sys.argv,
        )
        standalone.main()
        sys.argv = save


def generate_ts_static_parser(force: bool = False) -> None:
    """Generate static parser for TypeScript/JavaScript."""
    from lark.tools import standalone

    lark_ts_parser_path = os.path.join(_pycore_dir, "lark_ts_parser.py")
    if force or not os.path.exists(lark_ts_parser_path):
        sys.argv, save = (
            [
                "lark",
                os.path.join(_cur_dir, "ts.lark"),
                "-o",
                lark_ts_parser_path,
                "-c",
            ],
            sys.argv,
        )
        standalone.main()
        sys.argv = save


def gen_all_parsers() -> None:
    """Generate all parsers."""
    generate_static_parser(force=True)
    generate_ts_static_parser(force=True)
    print("Parsers generated.")


# Auto-generate parsers if missing (for developer setup)
_lark_jac_parser_path = os.path.join(_pycore_dir, "lark_jac_parser.py")
if not os.path.exists(_lark_jac_parser_path):
    print("Parser not present, generating for developer setup...", file=sys.stderr)
    try:
        gen_all_parsers()
    except Exception as e:
        print(f"Warning: Could not generate parser: {e}", file=sys.stderr)

__all__ = [
    "generate_static_parser",
    "generate_ts_static_parser",
    "gen_all_parsers",
]

if __name__ == "__main__":
    gen_all_parsers()



from _typeshed import Incomplete
from typing import Final

from reportlab.graphics.widgetbase import Widget
from reportlab.platypus import Flowable

__version__: Final[str]
adobe2codec: Incomplete

class CodeChartBase(Flowable):
    rows: Incomplete
    width: Incomplete
    height: Incomplete
    ylist: Incomplete
    xlist: Incomplete
    def calcLayout(self) -> None: ...
    def formatByte(self, byt): ...
    def drawChars(self, charList) -> None: ...
    def drawLabels(self, topLeft: str = "") -> None: ...

class SingleByteEncodingChart(CodeChartBase):
    codePoints: int
    faceName: Incomplete
    encodingName: Incomplete
    fontName: Incomplete
    charsPerRow: Incomplete
    boxSize: Incomplete
    hex: Incomplete
    rowLabels: Incomplete
    def __init__(
        self,
        faceName: str = "Helvetica",
        encodingName: str = "WinAnsiEncoding",
        charsPerRow: int = 16,
        boxSize: int = 14,
        hex: int = 1,
    ) -> None: ...
    def draw(self): ...

class KutenRowCodeChart(CodeChartBase):
    row: Incomplete
    codePoints: int
    boxSize: int
    charsPerRow: int
    rows: int
    rowLabels: Incomplete
    hex: int
    faceName: Incomplete
    encodingName: Incomplete
    fontName: Incomplete
    def __init__(self, row, faceName, encodingName) -> None: ...
    def makeRow(self, row): ...
    def draw(self) -> None: ...

class Big5CodeChart(CodeChartBase):
    row: Incomplete
    codePoints: int
    boxSize: int
    charsPerRow: int
    rows: int
    hex: int
    faceName: Incomplete
    encodingName: Incomplete
    rowLabels: Incomplete
    fontName: Incomplete
    def __init__(self, row, faceName, encodingName) -> None: ...
    def makeRow(self, row): ...
    def draw(self) -> None: ...

def hBoxText(msg, canvas, x, y, fontName) -> None: ...

class CodeWidget(Widget):
    x: int
    y: int
    width: int
    height: int
    def __init__(self) -> None: ...
    def draw(self): ...

def test() -> None: ...



"""PyCore - Bootstrap-critical Python core for Jac.

This package contains the minimal Python code required to bootstrap the Jac
compiler. Everything else in the jaclang codebase can be written in Jac.

Modules:
- unitree: Core AST definitions
- constant: Constants and token definitions
- codeinfo: Code location info for AST nodes
- jac_parser: Jac parser using Lark
- tsparser: TypeScript/JavaScript parser
- lark_jac_parser: Generated Lark parser for Jac
- lark_ts_parser: Generated Lark parser for TypeScript
- passes/: Bootstrap-critical compiler passes
- runtime: Runtime bootstrap infrastructure
- helpers: Utility functions
- log: Logging utilities
- modresolver: Module resolution utilities
- treeprinter: AST tree printing utilities
- settings: Configuration settings
- compiler: JacCompiler class (compilation singleton)
- program: JacProgram class (program state)
"""



from __future__ import annotations

import io
from typing_extensions import assert_type

import psycopg2.extensions
import psycopg2.extras
from psycopg2.extensions import make_dsn

# make_dsn
# --------

# (None) -> str
assert_type(make_dsn(), str)
assert_type(make_dsn(None), str)
assert_type(make_dsn(dsn=None), str)

# (bytes) -> bytes
assert_type(make_dsn(b""), bytes)
assert_type(make_dsn(dsn=b""), bytes)

# (bytes, **Kwargs) -> str
assert_type(make_dsn(b"", database=""), str)
assert_type(make_dsn(dsn=b"", database=""), str)

# (str, **OptionalKwargs) -> str
assert_type(make_dsn(""), str)
assert_type(make_dsn(dsn=""), str)
assert_type(make_dsn("", database=None), str)
assert_type(make_dsn(dsn="", database=None), str)


# connection.cursor
# -----------------

# (name?, None?, ...) -> psycopg2.extensions.cursor
conn = psycopg2.connect("test-conn")
assert_type(conn.cursor(), psycopg2.extensions.cursor)
assert_type(conn.cursor("test-cur"), psycopg2.extensions.cursor)
assert_type(conn.cursor("test-cur", None), psycopg2.extensions.cursor)
assert_type(conn.cursor("test-cur", cursor_factory=None), psycopg2.extensions.cursor)


# (name?, cursor_factory(), ...) -> custom_cursor
class MyCursor(psycopg2.extensions.cursor):
    pass


assert_type(conn.cursor("test-cur", cursor_factory=MyCursor), MyCursor)
assert_type(conn.cursor("test-cur", cursor_factory=lambda c, n: MyCursor(c, n)), MyCursor)

dconn = psycopg2.extras.DictConnection("test-dconn")
assert_type(dconn.cursor(), psycopg2.extras.DictCursor)
assert_type(dconn.cursor("test-dcur"), psycopg2.extras.DictCursor)
assert_type(dconn.cursor("test-dcur", None), psycopg2.extras.DictCursor)
assert_type(dconn.cursor("test-dcur", cursor_factory=None), psycopg2.extras.DictCursor)
assert_type(dconn.cursor("test-dcur", cursor_factory=MyCursor), MyCursor)

# file protocols
# --------------
cur = conn.cursor()
cur.copy_from(io.StringIO(), "table")



from datetime import date, datetime
from typing_extensions import assert_type

from dateutil.relativedelta import relativedelta


class MyDateTime(datetime):
    pass


d = MyDateTime.now()
x = d - relativedelta(days=1)
assert_type(x, MyDateTime)

d3 = datetime.today()
x3 = d3 - relativedelta(days=1)
assert_type(x3, datetime)

d2 = date.today()
x2 = d2 - relativedelta(days=1)
assert_type(x2, date)



import numpy as np
from networkx._typing import Array1D
from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = [
    "laplacian_spectrum",
    "adjacency_spectrum",
    "modularity_spectrum",
    "normalized_laplacian_spectrum",
    "bethe_hessian_spectrum",
]

@_dispatchable
def laplacian_spectrum(G: Graph[_Node], weight: str | None = "weight") -> Array1D[np.float64]: ...
@_dispatchable
def normalized_laplacian_spectrum(G: Graph[_Node], weight: str | None = "weight") -> Array1D[np.float64]: ...
@_dispatchable
def adjacency_spectrum(G: Graph[_Node], weight: str | None = "weight") -> Array1D[np.complex128]: ...
@_dispatchable
def modularity_spectrum(G: Graph[_Node]) -> Array1D[np.complex128]: ...
@_dispatchable
def bethe_hessian_spectrum(G: Graph[_Node], r: float | None = None) -> Array1D[np.float64]: ...



from _typeshed import Incomplete

from networkx.utils.backends import _dispatchable

from ..classes.digraph import DiGraph
from ..classes.graph import Graph
from ..classes.multidigraph import MultiDiGraph
from ..classes.multigraph import MultiGraph

__all__ = [
    "configuration_model",
    "directed_configuration_model",
    "expected_degree_graph",
    "havel_hakimi_graph",
    "directed_havel_hakimi_graph",
    "degree_sequence_tree",
    "random_degree_sequence_graph",
]

@_dispatchable
def configuration_model(deg_sequence, create_using=None, seed=None) -> MultiGraph[Incomplete]: ...
@_dispatchable
def directed_configuration_model(
    in_degree_sequence, out_degree_sequence, create_using=None, seed=None
) -> MultiDiGraph[Incomplete]: ...
@_dispatchable
def expected_degree_graph(w, seed=None, selfloops: bool = True) -> Graph[Incomplete]: ...
@_dispatchable
def havel_hakimi_graph(deg_sequence, create_using=None): ...
@_dispatchable
def directed_havel_hakimi_graph(in_deg_sequence, out_deg_sequence, create_using=None) -> DiGraph[Incomplete]: ...
@_dispatchable
def degree_sequence_tree(deg_sequence, create_using=None): ...
@_dispatchable
def random_degree_sequence_graph(sequence, seed=None, tries: int = 10) -> Graph[Incomplete]: ...

class DegreeSequenceRandomGraph:
    rng: Incomplete
    degree: Incomplete
    m: Incomplete
    dmax: Incomplete
    def __init__(self, degree, rng) -> None: ...
    remaining_degree: Incomplete
    graph: Incomplete
    def generate(self): ...
    def update_remaining(self, u, v, aux_graph=None) -> None: ...
    def p(self, u, v): ...
    def q(self, u, v): ...
    def suitable_edge(self): ...
    def phase1(self) -> None: ...
    def phase2(self) -> None: ...
    def phase3(self) -> None: ...



from collections.abc import Iterator
from typing import Any, Generic, TypeVar, type_check_only

from google.protobuf.descriptor import FieldDescriptor
from google.protobuf.internal.containers import RepeatedCompositeFieldContainer, RepeatedScalarFieldContainer
from google.protobuf.message import Message

_ContainerMessageT = TypeVar("_ContainerMessageT", bound=Message)
_ExtenderMessageT = TypeVar(
    "_ExtenderMessageT",
    bound=Message | RepeatedScalarFieldContainer[Any] | RepeatedCompositeFieldContainer[Any] | bool | float | str | bytes,
)

@type_check_only
class _ExtensionFieldDescriptor(FieldDescriptor, Generic[_ContainerMessageT, _ExtenderMessageT]): ...

class _ExtensionDict(Generic[_ContainerMessageT]):
    def __init__(self, extended_message: _ContainerMessageT) -> None: ...
    def __getitem__(
        self, extension_handle: _ExtensionFieldDescriptor[_ContainerMessageT, _ExtenderMessageT]
    ) -> _ExtenderMessageT: ...
    def __setitem__(
        self, extension_handle: _ExtensionFieldDescriptor[_ContainerMessageT, _ExtenderMessageT], value: _ExtenderMessageT
    ) -> None: ...
    def __delitem__(self, extension_handle: _ExtensionFieldDescriptor[_ContainerMessageT, _ExtenderMessageT]) -> None: ...
    def __contains__(self, extension_handle: _ExtensionFieldDescriptor[_ContainerMessageT, _ExtenderMessageT]) -> bool: ...
    def __iter__(self) -> Iterator[_ExtensionFieldDescriptor[_ContainerMessageT, Any]]: ...
    def __len__(self) -> int: ...



from typing import Final, Literal

from pymeeus.Angle import Angle
from pymeeus.Epoch import Epoch

PERIODIC_TERMS_LR_TABLE: Final[list[list[float]]]
PERIODIC_TERMS_B_TABLE: Final[list[list[float]]]

class Moon:
    @staticmethod
    def geocentric_ecliptical_pos(epoch: Epoch) -> tuple[Angle, Angle, float, Angle]: ...
    @staticmethod
    def apparent_ecliptical_pos(epoch: Epoch) -> tuple[Angle, Angle, float, Angle]: ...
    @staticmethod
    def apparent_equatorial_pos(epoch: Epoch) -> tuple[Angle, Angle, float, Angle]: ...
    @staticmethod
    def longitude_mean_ascending_node(epoch: Epoch) -> Angle: ...
    @staticmethod
    def longitude_true_ascending_node(epoch: Epoch) -> Angle: ...
    @staticmethod
    def longitude_mean_perigee(epoch: Epoch) -> Angle: ...
    @staticmethod
    def illuminated_fraction_disk(epoch: Epoch) -> float: ...
    @staticmethod
    def position_bright_limb(epoch: Epoch) -> Angle: ...
    @staticmethod
    def moon_phase(epoch: Epoch, target: Literal["new", "first", "full", "last"] = "new") -> Epoch: ...
    @staticmethod
    def moon_perigee_apogee(epoch: Epoch, target: Literal["perigee", "apogee"] = "perigee") -> tuple[Epoch, Angle]: ...
    @staticmethod
    def moon_passage_nodes(epoch: Epoch, target: Literal["ascending", "descending"] = "ascending") -> Epoch: ...
    @staticmethod
    def moon_maximum_declination(epoch: Epoch, target: Literal["northern", "southern"] = "northern") -> tuple[Epoch, Angle]: ...
    @staticmethod
    def moon_librations(epoch: Epoch) -> tuple[Angle, Angle, Angle, Angle, Angle, Angle]: ...
    @staticmethod
    def moon_position_angle_axis(epoch: Epoch) -> Angle: ...

def main() -> None: ...



from collections.abc import Mapping, Sequence
from typing import Final

DEFAULT_DOCKER_API_VERSION: Final[str]
MINIMUM_DOCKER_API_VERSION: Final[str]
DEFAULT_TIMEOUT_SECONDS: Final[int]
STREAM_HEADER_SIZE_BYTES: Final[int]
CONTAINER_LIMITS_KEYS: Final[Sequence[str]]
DEFAULT_HTTP_HOST: Final[str]
DEFAULT_UNIX_SOCKET: Final[str]
DEFAULT_NPIPE: Final[str]
BYTE_UNITS: Final[Mapping[str, int]]
INSECURE_REGISTRY_DEPRECATION_WARNING: Final[str]
IS_WINDOWS_PLATFORM: Final[bool]
WINDOWS_LONGPATH_PREFIX: Final[str]
DEFAULT_USER_AGENT: Final[str]
DEFAULT_NUM_POOLS: Final[int]
DEFAULT_NUM_POOLS_SSH: Final[int]
DEFAULT_MAX_POOL_SIZE: Final[int]
DEFAULT_DATA_CHUNK_SIZE: Final[int]
DEFAULT_SWARM_ADDR_POOL: Final[Sequence[str]]
DEFAULT_SWARM_SUBNET_SIZE: Final[int]



from typing import overload
from typing_extensions import deprecated

from ._typing import GeoArray, OptGeoArrayLike, OptGeoArrayLikeSeq
from .geometry.base import BaseGeometry
from .lib import Geometry

__all__ = [
    "coverage_union",
    "coverage_union_all",
    "difference",
    "disjoint_subset_union",
    "disjoint_subset_union_all",
    "intersection",
    "intersection_all",
    "symmetric_difference",
    "symmetric_difference_all",
    "unary_union",
    "union",
    "union_all",
]

@overload
def difference(a: Geometry, b: Geometry, grid_size: float | None = None, **kwargs) -> BaseGeometry: ...
@overload
def difference(a: None, b: Geometry | None, grid_size: float | None = None, **kwargs) -> None: ...
@overload
def difference(a: Geometry | None, b: None, grid_size: float | None = None, **kwargs) -> None: ...
@overload
def difference(a: OptGeoArrayLikeSeq, b: OptGeoArrayLike, grid_size: float | None = None, **kwargs) -> GeoArray: ...
@overload
def difference(a: OptGeoArrayLike, b: OptGeoArrayLikeSeq, grid_size: float | None = None, **kwargs) -> GeoArray: ...
@overload
def intersection(a: Geometry, b: Geometry, grid_size: float | None = None, **kwargs) -> BaseGeometry: ...
@overload
def intersection(a: None, b: Geometry | None, grid_size: float | None = None, **kwargs) -> None: ...
@overload
def intersection(a: Geometry | None, b: None, grid_size: float | None = None, **kwargs) -> None: ...
@overload
def intersection(a: OptGeoArrayLikeSeq, b: OptGeoArrayLike, grid_size: float | None = None, **kwargs) -> GeoArray: ...
@overload
def intersection(a: OptGeoArrayLike, b: OptGeoArrayLikeSeq, grid_size: float | None = None, **kwargs) -> GeoArray: ...
@overload
def intersection_all(geometries: OptGeoArrayLike, axis: None = None, **kwargs) -> BaseGeometry: ...
@overload
def intersection_all(geometries: OptGeoArrayLikeSeq, axis: int, **kwargs) -> BaseGeometry | GeoArray: ...
@overload
def symmetric_difference(a: Geometry, b: Geometry, grid_size: float | None = None, **kwargs) -> BaseGeometry: ...
@overload
def symmetric_difference(a: None, b: Geometry | None, grid_size: float | None = None, **kwargs) -> None: ...
@overload
def symmetric_difference(a: Geometry | None, b: None, grid_size: float | None = None, **kwargs) -> None: ...
@overload
def symmetric_difference(a: OptGeoArrayLikeSeq, b: OptGeoArrayLike, grid_size: float | None = None, **kwargs) -> GeoArray: ...
@overload
def symmetric_difference(a: OptGeoArrayLike, b: OptGeoArrayLikeSeq, grid_size: float | None = None, **kwargs) -> GeoArray: ...
@overload
@deprecated("symmetric_difference_all behaves incorrectly and will be removed in a future version.")
def symmetric_difference_all(geometries: OptGeoArrayLike, axis: None = None, **kwargs) -> BaseGeometry: ...
@overload
@deprecated("symmetric_difference_all behaves incorrectly and will be removed in a future version.")
def symmetric_difference_all(geometries: OptGeoArrayLikeSeq, axis: int, **kwargs) -> BaseGeometry | GeoArray: ...
@overload
def union(a: Geometry, b: Geometry, grid_size: float | None = None, **kwargs) -> BaseGeometry: ...
@overload
def union(a: None, b: Geometry | None, grid_size: float | None = None, **kwargs) -> None: ...
@overload
def union(a: Geometry | None, b: None, grid_size: float | None = None, **kwargs) -> None: ...
@overload
def union(a: OptGeoArrayLikeSeq, b: OptGeoArrayLike, grid_size: float | None = None, **kwargs) -> GeoArray: ...
@overload
def union(a: OptGeoArrayLike, b: OptGeoArrayLikeSeq, grid_size: float | None = None, **kwargs) -> GeoArray: ...
@overload
def union_all(geometries: OptGeoArrayLike, grid_size: float | None = None, axis: None = None, **kwargs) -> BaseGeometry: ...
@overload
def union_all(
    geometries: OptGeoArrayLikeSeq, grid_size: float | None = None, *, axis: int, **kwargs
) -> BaseGeometry | GeoArray: ...
@overload
def union_all(geometries: OptGeoArrayLikeSeq, grid_size: float | None, axis: int, **kwargs) -> BaseGeometry | GeoArray: ...

unary_union = union_all

@overload
def coverage_union(a: OptGeoArrayLike, b: OptGeoArrayLike, *, axis: None = None, **kwargs) -> BaseGeometry: ...
@overload
def coverage_union(a: OptGeoArrayLike, b: OptGeoArrayLike, *, axis: int, **kwargs) -> BaseGeometry | GeoArray: ...
@overload
def coverage_union_all(geometries: OptGeoArrayLike, axis: None = None, **kwargs) -> BaseGeometry: ...
@overload
def coverage_union_all(geometries: OptGeoArrayLikeSeq, axis: int, **kwargs) -> BaseGeometry | GeoArray: ...
def disjoint_subset_union(a: OptGeoArrayLike, b: OptGeoArrayLike, **kwargs) -> BaseGeometry | GeoArray: ...
@overload
def disjoint_subset_union_all(geometries: OptGeoArrayLike, *, axis: None = None, **kwargs) -> BaseGeometry: ...
@overload
def disjoint_subset_union_all(geometries: OptGeoArrayLikeSeq, *, axis: int, **kwargs) -> BaseGeometry | GeoArray: ...



from typing import Literal, NoReturn, overload

import _win32typing
from win32.lib.pywintypes import error as error

def GetConsoleProcessList() -> tuple[int, ...]: ...
def CreateConsoleScreenBuffer(
    DesiredAccess=..., ShareMode=..., SecurityAttributes: _win32typing.PySECURITY_ATTRIBUTES | None = ..., Flags=...
) -> _win32typing.PyConsoleScreenBuffer: ...
def GetConsoleDisplayMode(): ...
def AttachConsole(ProcessId) -> None: ...
def AllocConsole() -> None: ...
def FreeConsole() -> None: ...
def GetConsoleCP(): ...
def GetConsoleOutputCP(): ...
def SetConsoleCP(CodePageId) -> None: ...
def SetConsoleOutputCP(CodePageID) -> None: ...
def GetConsoleSelectionInfo(): ...
def AddConsoleAlias(Source, Target, ExeName) -> None: ...
def GetConsoleAliases(ExeName: str) -> str: ...
def GetConsoleAliasExes(): ...
def GetConsoleWindow(): ...
def GetNumberOfConsoleFonts(): ...
def SetConsoleTitle(ConsoleTitle: str) -> None: ...
def GetConsoleTitle(): ...
@overload
def GenerateConsoleCtrlEvent(CtrlEvent: Literal[1], ProcessGroupId: Literal[0] = 0) -> NoReturn: ...
@overload
def GenerateConsoleCtrlEvent(CtrlEvent: Literal[0, 1], ProcessGroupId: int) -> None: ...
def GetStdHandle(StdHandle: int) -> _win32typing.PyConsoleScreenBuffer: ...

ATTACH_PARENT_PROCESS: int
BACKGROUND_BLUE: int
BACKGROUND_GREEN: int
BACKGROUND_INTENSITY: int
BACKGROUND_RED: int
COMMON_LVB_GRID_HORIZONTAL: int
COMMON_LVB_GRID_LVERTICAL: int
COMMON_LVB_GRID_RVERTICAL: int
COMMON_LVB_LEADING_BYTE: int
COMMON_LVB_REVERSE_VIDEO: int
COMMON_LVB_TRAILING_BYTE: int
COMMON_LVB_UNDERSCORE: int
CONSOLE_FULLSCREEN: int
CONSOLE_FULLSCREEN_HARDWARE: int
CONSOLE_FULLSCREEN_MODE: int
CONSOLE_MOUSE_DOWN: int
CONSOLE_MOUSE_SELECTION: int
CONSOLE_NO_SELECTION: int
CONSOLE_SELECTION_IN_PROGRESS: int
CONSOLE_SELECTION_NOT_EMPTY: int
CONSOLE_TEXTMODE_BUFFER: int
CONSOLE_WINDOWED_MODE: int
CTRL_BREAK_EVENT: int
CTRL_C_EVENT: int
ENABLE_ECHO_INPUT: int
ENABLE_LINE_INPUT: int
ENABLE_MOUSE_INPUT: int
ENABLE_PROCESSED_INPUT: int
ENABLE_PROCESSED_OUTPUT: int
ENABLE_WINDOW_INPUT: int
ENABLE_WRAP_AT_EOL_OUTPUT: int
FOCUS_EVENT: int
FOREGROUND_BLUE: int
FOREGROUND_GREEN: int
FOREGROUND_INTENSITY: int
FOREGROUND_RED: int
KEY_EVENT: int
LOCALE_USER_DEFAULT: int
MENU_EVENT: int
MOUSE_EVENT: int
PyCOORDType = _win32typing.PyCOORD
PyConsoleScreenBufferType = _win32typing.PyConsoleScreenBuffer
PyINPUT_RECORDType = _win32typing.PyINPUT_RECORD
PySMALL_RECTType = _win32typing.PySMALL_RECT
STD_ERROR_HANDLE: int
STD_INPUT_HANDLE: int
STD_OUTPUT_HANDLE: int
WINDOW_BUFFER_SIZE_EVENT: int



from _typeshed import Incomplete

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable
from numpy.random import RandomState

__all__ = ["kernighan_lin_bisection"]

@_dispatchable
def kernighan_lin_bisection(
    G: Graph[_Node],
    partition: tuple[Incomplete] | None = None,
    max_iter: int = 10,
    weight: str = "weight",
    seed: int | RandomState | None = None,
): ...



"""Preconfigured converters for the stdlib json."""

from cattrs.preconf.json import JsonConverter, configure_converter, make_converter

__all__ = ["configure_converter", "JsonConverter", "make_converter"]



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import tensorflow.core.framework.tensor_shape_pb2
import tensorflow.core.framework.types_pb2

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class ResourceHandleProto(google.protobuf.message.Message):
    """Protocol buffer representing a handle to a tensorflow resource. Handles are
    not valid across executions, but can be serialized back and forth from within
    a single run.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class DtypeAndShape(google.protobuf.message.Message):
        """Protocol buffer representing a pair of (data type, tensor shape)."""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        DTYPE_FIELD_NUMBER: builtins.int
        SHAPE_FIELD_NUMBER: builtins.int
        dtype: tensorflow.core.framework.types_pb2.DataType.ValueType
        """Data type of the tensor."""
        @property
        def shape(self) -> tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto:
            """Shape of the tensor."""

        def __init__(
            self,
            *,
            dtype: tensorflow.core.framework.types_pb2.DataType.ValueType | None = ...,
            shape: tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["shape", b"shape"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["dtype", b"dtype", "shape", b"shape"]) -> None: ...

    DEVICE_FIELD_NUMBER: builtins.int
    CONTAINER_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    HASH_CODE_FIELD_NUMBER: builtins.int
    MAYBE_TYPE_NAME_FIELD_NUMBER: builtins.int
    DTYPES_AND_SHAPES_FIELD_NUMBER: builtins.int
    device: builtins.str
    """Unique name for the device containing the resource."""
    container: builtins.str
    """Container in which this resource is placed."""
    name: builtins.str
    """Unique name of this resource."""
    hash_code: builtins.int
    """Hash code for the type of the resource. Is only valid in the same device
    and in the same execution.
    """
    maybe_type_name: builtins.str
    """For debug-only, the name of the type pointed to by this handle, if
    available.
    """
    @property
    def dtypes_and_shapes(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ResourceHandleProto.DtypeAndShape]:
        """Data types and shapes for the underlying resource."""

    def __init__(
        self,
        *,
        device: builtins.str | None = ...,
        container: builtins.str | None = ...,
        name: builtins.str | None = ...,
        hash_code: builtins.int | None = ...,
        maybe_type_name: builtins.str | None = ...,
        dtypes_and_shapes: collections.abc.Iterable[global___ResourceHandleProto.DtypeAndShape] | None = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "container",
            b"container",
            "device",
            b"device",
            "dtypes_and_shapes",
            b"dtypes_and_shapes",
            "hash_code",
            b"hash_code",
            "maybe_type_name",
            b"maybe_type_name",
            "name",
            b"name",
        ],
    ) -> None: ...

global___ResourceHandleProto = ResourceHandleProto



"""Basic Logging Module for Jac Compiler."""

import logging

logging.basicConfig(
    # level=logging.INFO, format="%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    level=logging.INFO,
    format="%(levelname)s - %(message)s",
)

__all__ = ["logging"]
